#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# editable list dialog

"""  editable list dialog """

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from taurus.qt.qtgui.util.ui import UILoadable


import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


@UILoadable(with_ui='ui')
class OrderDlg(Qt.QDialog):
    """  editable list dialog with order of channels"""

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QDialog.__init__(self, parent)
        self.loadUi()
        #: (:obj:`bool`) dirty flag
        self.dirty = False
        #: (:obj:`list` <:obj:`str`>) all channels channels
        self.channels = []
        #: (:obj:`list` <:obj:`str`>) selected channels
        self.selected = []
        #: (:obj:`bool`) order only selected channels
        self.onlyselected = False

    def __setDirty(self):
        """ sets dirty to True
        """
        self.dirty = True

    def createGUI(self):
        """ creates GUI
        """
        self.ui.closePushButton = self.ui.closeButtonBox.button(
            Qt.QDialogButtonBox.Close)

        self.ui.closeButtonBox.button(Qt.QDialogButtonBox.Close).hide()
        self.ui.upPushButton = self.ui.upDownButtonBox.addButton(
            "&Up", Qt.QDialogButtonBox.ActionRole)
        self.ui.downPushButton = self.ui.upDownButtonBox.addButton(
            "&Down", Qt.QDialogButtonBox.ActionRole)
        self.ui.selPushButton = self.ui.closeButtonBox.addButton(
            "&Only Selected", Qt.QDialogButtonBox.ActionRole)
        self.ui.selPushButton.setCheckable(True)
        self.ui.selPushButton.setChecked(self.onlyselected)

        if self.channels:
            item = self.channels[0]
        else:
            item = None
        self.selected = list(set(self.channels) & set(self.selected))

        self.__populateList(item)
        self.ui.upPushButton.clicked.connect(self.__up)
        self.ui.downPushButton.clicked.connect(self.__down)
        self.ui.selPushButton.clicked.connect(self.__setfilter)

        self.ui.closeButtonBox.button(
            Qt.QDialogButtonBox.Close).clicked.connect(self.accept)
        self.ui.closePushButton.show()

    def __populateList(self, selectedChannel=None):
        """ populates the group list

        :param selectedChannel: selected channel
        :type selectedChannel: :obj:`str`
        """
        selected = None
        self.ui.listWidget.clear()
        for ch in self.channels:
            if self.onlyselected:
                if ch not in self.selected:
                    continue
            item = Qt.QListWidgetItem(str(ch))
            self.ui.listWidget.addItem(item)
            if selectedChannel is not None and selectedChannel == ch:
                selected = item
            if selected is not None:
                selected.setSelected(True)
                self.ui.listWidget.setCurrentItem(selected)

    def __currentName(self):
        """ provides a name of the currently selected channel

        :returns: name of the currently selected channel
        :rtype: :obj:`str`
        """
        name = None
        row = self.ui.listWidget.currentRow()
        if not self.onlyselected:
            if len(self.channels) > row and row >= 0:
                name = self.channels[row]
        else:
            index = -1
            for i, channel in enumerate(self.channels):
                if channel in self.selected:
                    index += 1
                if row == index:
                    name = self.channels[i]
                    break
        return name

    @Qt.pyqtSlot()
    def __up(self):
        """ moves currently selected channel up
        """
        name = self.__currentName()
        if not name:
            return
        ni = self.channels.index(name)
        if ni > 0:
            nim1 = ni - 1
            if self.onlyselected:
                while nim1 and self.channels[nim1] not in self.selected:
                    nim1 -= 1
            self.channels[ni], self.channels[nim1] = \
                self.channels[nim1], self.channels[ni]
            self.__populateList(name)
            self.__setDirty()

    @Qt.pyqtSlot()
    def __down(self):
        """ moves currently selected channel down
        """
        name = self.__currentName()
        if not name:
            return
        ni = self.channels.index(name)
        if ni < len(self.channels) - 1:
            nip1 = ni + 1
            if self.onlyselected:
                while nip1 < len(self.channels) - 1 \
                        and self.channels[nip1] not in self.selected:
                    nip1 += 1
            self.channels[ni], self.channels[nip1] = \
                self.channels[nip1], self.channels[ni]
            self.__populateList(name)
            self.__setDirty()

    @Qt.pyqtSlot()
    def __setfilter(self):
        """ sets only selected channel filter according to selPushButton
        """
        self.onlyselected = bool(self.ui.selPushButton.isChecked())
        name = self.__currentName()
        self.__populateList(name)



#!/usr/bin/env python
#   This file is part of nexdatas - Tanog Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# selactable tab

""" detector component tab """

import json
import fnmatch

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from .Element import DSElement, CPElement, CP, DS
from .ElementModel import ElementModel
from .Frames import Frames
from .DynamicTools import DynamicTools

from .Views import CheckerView

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


class Detectors(Qt.QObject):
    """ detector component tab widget
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()

    def __init__(self, ui, state=None, userView=CheckerView, rowMax=0,
                 simpleMode=0, fontSize=11):
        """ constructor

        :param ui: ui instance
        :type ui: :class:`taurus.qt.qtgui.util.ui.__UI`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param userView: widget for description optional components
        :type userView: :class:`taurus.qt.Qt.QWidget`
        :param rowMax: maximal row number in component column view
        :type rowMax: :obj:`int`
        :param simpleMode: if simple display mode: \
                           `1` for negative hidden, `2` for negative disable
        :type simpleMode: :obj:`int`
        :param fontSize: font size in component column view
        :type fontSize: :obj:`int`
        """

        Qt.QObject.__init__(self)

        #: (:class:`taurus.qt.qtgui.util.ui.__UI`) ui instance
        self.ui = ui
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = state
        #: (:class:`taurus.qt.Qt.QWidget`) \
        #:     widget for description optional components
        self.userView = userView
        #: (:obj:`int`) maximal row number in component column view
        self.rowMax = rowMax
        #: (:obj:`int`) font size for in component column view
        self.fontSize = fontSize
        #: (:obj:`bool`) if simple view mode
        self.__simpleMode = simpleMode
        #: (:class:`taurus.qt.Qt.QLayout`) component layout
        self.glayout = None
        #: (:obj:`str`)  JSON dictionary as \
        #: with nested list frames [[[[group_name, group_id ], ...], ...], ...]
        self.frames = None
        #: (:obj:`str`)  JSON dictionary as \
        #: (:obj:`dict` <:obj:`int`, :obj:`list` <:obj:`str`> >) \
        #:    component group filters
        self.mgroups = None
        #: (:obj:`dict` <:obj:`int`, :class:`taurus.qt.Qt.QWidget`>) \
        #:    (OPTIONAL, MANDATORY, OTHERS) component views
        self.groups = {}
        #: (:obj:`dict` <:obj:`int`, :class:`taurus.qt.Qt.QWidget`>) \
        #:    (OPTIONAL, MANDATORY, OTHERS) component views
        self.views = {}
        #: (:obj:`list` <:class:`taurus.qt.Qt.QFrame`>) \
        #:  list of component group frames
        self.mframes = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QLayout`>) \
        #:  list of column layouts
        self.column_layouts = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QLayout`>) \
        #:  list of  group layouts
        self.group_layouts = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QLayout`>) \
        #:  list of widget layouts
        self.auto_layouts = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QGroupBox`>) \
        #:  list of component group boxes
        self.groupboxes = []
        #: (:obj:`list` <:class:`nxsselector.ElementModel.ElementModel`>) \
        #:    element model list
        self.models = []

    def updateGroups(self):
        """ filters component/datasource selection state
        """

        self.groups = {}
        ucp = set()
        uds = set()
        try:
            mgroups = json.loads(self.mgroups)
        except Exception:
            mgroups = {}
        for k, gr in mgroups.items():
            if int(k) in self.__availableGroups():
                group = []
                for elem in gr:
                    if isinstance(elem, list) and elem:
                        ielem = elem[0]
                    else:
                        ielem = elem
                    cpfiltered = fnmatch.filter(
                        self.state.cpgroup.keys(),
                        ielem)
                    dsfiltered = fnmatch.filter(
                        self.state.dsgroup.keys(),
                        ielem)
                    filtered = set(cpfiltered)
                    filtered.update(dsfiltered)
                    for felem in filtered:
                        if felem in self.state.avcplist:
                            group.append(
                                CPElement(felem, self.state))
                            ucp.add(felem)
                        else:
                            group.append(
                                DSElement(felem, self.state))
                            uds.add(felem)
                if group:
                    if int(k) not in self.groups:
                        self.groups[int(k)] = []
                    self.groups[int(k)].extend(group)

        for ds in self.state.dsgroup.keys():
            if ds not in uds and ds not in ucp:
                if DS not in self.groups:
                    self.groups[DS] = []
                if ds in self.state.avcplist:
                    self.groups[DS].append(CPElement(ds, self.state))
                    ucp.add(ds)
                else:
                    self.groups[DS].append(DSElement(ds, self.state))
                    uds.add(ds)
        for cp in self.state.cpgroup.keys():
            if cp not in ucp and cp not in uds \
                    and cp not in self.state.mcplist \
                    and cp not in self.state.acplist:
                if CP not in self.groups:
                    self.groups[CP] = []
                self.groups[CP].append(CPElement(cp, self.state))

    def __availableGroups(self):
        """ provides a set of Frames with available groups

        :returns: set of Frames with available groups
        :rtype: :obj:`set` <:obj:`nxsselector.Frames.Frames`>
        """
        res = set()
        try:
            frames = Frames(self.frames)
            for frame in frames:
                for column in frame:
                    for group in column:
                        res.add(group[1])
        except Exception:
            pass
        return res

    def __clearFrames(self):
        """ clears component frames
        """
        DynamicTools.cleanupObjects(self.models, "model")
        if self.views:
            views = list(self.views.values())
            DynamicTools.cleanupWidgets(views, "views")
            self.views = {}
        DynamicTools.cleanupObjects(self.auto_layouts, "auto")
        DynamicTools.cleanupWidgets(self.groupboxes, "groupbox")
        DynamicTools.cleanupObjects(self.group_layouts, "group")
        DynamicTools.cleanupObjects(self.column_layouts, "column")
        DynamicTools.cleanupFrames(self.mframes, "frames")
        DynamicTools.cleanupLayoutWithItems(self.glayout)

    def __calcMaxRowNumbers(self):
        """ calculates row numbers for groups

        :returns:  a dictionry with rownumbers
        :rtype: :obj:`dict`< :obj:`int`, :obj:`int`>
        """
        maxrownumbers = {}
        gpsizes = dict([(gk, len(gr)) for gk, gr in self.groups.items()])
        for frame in json.loads(self.frames):
            for column in frame:
                clm = [gr[1] for gr in column]
                if len(clm) > 1:
                    gpsum = sum([(gpsizes[gr[1]] if gr[1] in gpsizes else 0)
                                 for gr in column])
                    for gr in column:
                        la = float(
                            gpsizes[gr[1]] if gr[1] in gpsizes else 0) / \
                            max(gpsum, 1)
                        maxrownumbers[gr[1]] =  \
                            max(1, int(la * (self.rowMax - 2*len(clm) + 2)))
                else:
                    maxrownumbers[column[0][1]] = self.rowMax
        return maxrownumbers

    def createGUI(self):
        """ creates widget GUI
        """

        self.__clearFrames()
        self.glayout = Qt.QHBoxLayout(self.ui.detectors)

        frames = Frames(self.frames, DS in self.groups, CP in self.groups)
        maxrownumbers = self.__calcMaxRowNumbers()
        for frame in frames:
            mframe = Qt.QFrame(self.ui.detectors)
            self.mframes.append(mframe)
            mframe.setFrameShape(Qt.QFrame.StyledPanel)
            mframe.setFrameShadow(Qt.QFrame.Raised)
            layout_columns = Qt.QHBoxLayout(mframe)
            self.column_layouts.append(layout_columns)

            for column in frame:
                layout_groups = Qt.QVBoxLayout()
                self.group_layouts.append(layout_groups)

                for group in column:
                    hide = False
                    try:
                        ig = int(group[1])
                        if (self.__simpleMode & 1) and ig < 0:
                            hide = True
                    except Exception:
                        pass
                    if not hide:
                        mgroup = Qt.QGroupBox(mframe)
                        self.groupboxes.append(mgroup)
                        mgroup.setTitle(group[0])
                        layout_auto = Qt.QGridLayout(mgroup)
                        self.auto_layouts.append(layout_auto)
                        mview = self.userView(mgroup)
                        if hasattr(mview, "rowMax"):
                            if group[1] in maxrownumbers.keys():
                                mview.rowMax = maxrownumbers[group[1]]
                            else:
                                mview.rowMax = self.rowMax
                        if hasattr(mview, "fontSize"):
                            mview.fontSize = self.fontSize

                        layout_auto.addWidget(mview, 0, 0, 1, 1)
                        self.views[group[1]] = mview
                        layout_groups.addWidget(mgroup)
                if layout_groups.count():
                    layout_columns.addLayout(layout_groups)
            if layout_columns.count():
                self.glayout.addWidget(mframe)
            else:
                mframe.hide()
        self.ui.detectors.update()
        if self.ui.tabWidget.currentWidget() == self.ui.detectors:
            self.ui.detectors.show()

    def setModels(self):
        """ sets view models
        """
        for k in self.views.keys():
            if k in self.groups.keys():
                md = ElementModel(self.groups[k])
            else:
                md = ElementModel([])
            self.models.append(md)
            try:
                ig = int(k)
                if (self.__simpleMode & 2) and ig < 0:
                    md.enable = False
                    md.disEnable = False
            except Exception:
                pass
            self.views[k].setModel(md)

            md.componentChecked.connect(self.updateViews)
            md.dirty.connect(self.setDirty)

    @Qt.pyqtSlot()
    def setDirty(self):
        """ emits the `dirty` signal
        """
        self.dirty.emit()

    def reset(self):
        """ recreates widget GUI
        """
        logger.debug("reset views")
        self.updateGroups()
        self.createGUI()
        self.setModels()
        self.updateViews()
        logger.debug("reset views end")

    @Qt.pyqtSlot()
    def updateViews(self):
        """ resets frame views
        """
        logger.debug("update views")
        for vw in self.views.values():
            vw.reset()

        logger.debug("update views end")



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# command thread

""" command thread """

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt


class CommandThread(Qt.QThread):
    """ thread which executes a list of commands
    """
    #: (:class:`taurus.qt.Qt.pyqtSignal`) finisched signal
    finished = Qt.pyqtSignal()

    def __init__(self, instance, commands, parent):
        """ thread contructor

        :param instance: command instance
        :type instance: :obj:`instanceobj` or :obj:`type`
        :param commands: a list of commands
        :type commands: :obj:`list` <:obj:`str`>
        :param parent: thread parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """

        Qt.QThread.__init__(self, parent)
        #: (:obj:`instanceobj` or :obj:`type`) command instance
        self.instance = instance
        #: (:obj:`list` <:obj:`str`>) a list of commands
        self.commands = list(commands)
        #: (:obj:`Exception`) error thrown by the executed command
        self.error = None

    def run(self):
        """ run thread method
        """
        try:
            for cmd in self.commands:
                getattr(self.instance, cmd)()
        except Exception as e:
            self.error = e
        finally:
            self.finished.emit()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
# import unittest
# import os
import sys
# import random
# import struct
# import binascii
# # import time
# # import threading
# # import PyTango
# # import json
# import nxsselector
# from nxsselector import Selector

import taurus.qt.qtgui.application
Application = taurus.qt.qtgui.application.TaurusApplication

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO


if sys.version_info > (3,):
    unicode = str
    long = int


# comp_available tesQt
# \brief It tests XMLConfigurator
def testhelp():

    helps = ['--help', '-h']
    for hl in helps:
        print(hl)
        old_argv = sys.argv
        sys.argv = ['nxselector', hl]
        try:
            print("WW")
            app = Application(
                sys.argv,
                # cmd_line_parser=parser,
                app_name="NXS Component Selector",
                #            app_version=__version__,
                org_domain="desy.de",
                org_name="DESY")
            print("WW2")
            # Selector.main()
        except SystemExit as e:
            print("EXIT")
            print(e)
        except Exception as e:
            print("EXCEP")
            print(e)

        



if __name__ == '__main__':
    testhelp()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# Element

""" device Model """

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)

#: (:obj:`int`) datasource=0, component=1  element types
DS, CP = range(2)


class Element(object):
    """ element class
    """

    def __init__(self, name, eltype, state):
        """ constructor

        :param name: element name
        :type name: :obj:`str`
        :param eltype: element type, i.e. DS=0 or CP=1
        :type eltype: :obj:`int`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        """
        #: (:obj:`str`) element name
        self.name = name
        #: (:obj:`int`) element type, i.e. DS=0 or CP=1
        self.eltype = eltype
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = state

    def __getChecked(self):
        """ getter for checked flag

        :returns: if element is checked
        :rtype: :obj:`bool` or `None`
        """
        return self._getChecked()

    def __setChecked(self, status):
        """ setter for checked flag

        :param status: check status
        :type: :obj:`bool` or `None`
        """
        self._setChecked(status)

    def _getChecked(self):
        """ virtual getter for checked flag

        :returns: if element is checked
        :rtype: :obj:`bool` or `None`
        """
        pass

    def _setChecked(self, _):
        """ virtual setter for checked flag

        :param status: check status
        :type: :obj:`bool` or `None`
        """
        pass

    checked = property(__getChecked, __setChecked,
                       doc='check status')

    def __getEnable(self):
        """ getter for enable flag

        :returns: if element is enable
        :rtype: :obj:`bool`
        """
        return self._getEnable()

    def _getEnable(self):
        """ virtual getter for enable flag

        :returns: if element is enable
        :rtype: :obj:`bool`
        """
        return True

    enable = property(__getEnable, doc='check status')

    def __getDisplay(self):
        """ getter for display flag

        :returns: if element should be displayed
        :rtype: :obj:`bool`
        """
        return self._getDisplay()

    def __setDisplay(self, status):
        """ setter for display flag

        :param status: display status
        :type: :obj:`bool` or `None`
        """
        self._setDisplay(status)

    def _getDisplay(self):
        """ virtual getter for display flag

        :returns: if element should be displayed
        :rtype: :obj:`bool`
        """
        pass

    def _setDisplay(self, _):
        """ virtual setter for display flag

        :param status: display status
        :type: :obj:`bool` or `None`
        """
        pass

    display = property(__getDisplay, __setDisplay,
                       doc='check status')

    def __str__(self):
        return (self.name, self.eltype, self.state)

    def __repr__(self):
        return "(%s,%s)" % (self.name, self.eltype)


class GElement(Element):
    """ group element class
    """

    def __init__(self, name, eltype, state, dct):
        """ constructor

        :param name: element name
        :type name: :obj:`str`
        :param eltype: element type, i.e. DS=0 or CP=1
        :type eltype: :obj:`int`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param dct: element selection group
        :type dct: :obj:`dict` <:obj:`str`, :obj:`bool` or `None`>
        """
        super(GElement, self).__init__(name, eltype, state)
        #: (:obj:`dict` <:obj:`str`, :obj:`bool` or `None`> ) \
        #:     element selection group
        self.group = dct

    def _getChecked(self):
        """ getter for checked flag

        :returns: if element is checked
        :rtype: :obj:`bool` or `None`
        """
        if self.name in self.group.keys():
            return self.group[self.name]
        return False

    def _setChecked(self, status):
        """ setter for checked flag

        :param status: check status
        :type: :obj:`bool` or `None`
        """
        logger.debug("Changed: %s to %s" % (self.name, status))
        self.group[self.name] = bool(status)


class DSElement(Element):
    """ datasource element class
    """

    def __init__(self, name, state):
        """ constructor

        :param name: element name
        :type name: :obj:`str`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        """
        super(DSElement, self).__init__(name, DS, state)

    def _getChecked(self):
        """ getter for checked flag

        :returns: if element is checked
        :rtype: :obj:`bool` or `None`
        """
        res = self.state.dsgroup
        timers = self.state.timers or []
        if self.name in timers:
            return True
        if self.name in res.keys():
            return res[self.name]
        return False

    def _setChecked(self, status):
        """ setter for checked flag

        :param status: check status
        :type: :obj:`bool` or `None`
        """
        dc = self.state.dsgroup
        dc[self.name] = status
        self.state.ddsdirty = True
        if not status:
            nd = self.state.nodisplay
            if self.name in nd:
                nd.remove(self.name)

    def _getDisplay(self):
        """ getter for display flag

        :returns: if element should be displayed
        :rtype: :obj:`bool`
        """
        res = self.state.dsgroup
        nd = self.state.nodisplay
        if self.name not in nd:
            if self.name in res.keys():
                return res[self.name]
        return False

    def _setDisplay(self, status):
        """ setter for display flag

        :param status: display status
        :type: :obj:`bool` or `None`
        """
        dc = self.state.dsgroup
        nd = self.state.nodisplay
        timers = self.state.timers or []
        if self.name in dc.keys():
            if self.name in nd:
                if status:
                    nd.remove(self.name)
                    dc[self.name] = True
            else:
                if not status:
                    if self.name in timers:
                        dc[self.name] = False
                    else:
                        nd.append(self.name)
                else:
                    dc[self.name] = True

        else:
            if self.name in nd:
                nd.remove(self.name)


class CPElement(Element):
    """ datasource element class
    """

    def __init__(self, name, state, group=None):
        """ constructor

        :param name: element name
        :type name: :obj:`str`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param group: element selection group
        :type group: :obj:`dict` <:obj:`str`, :obj:`bool` or `None`>
        """
        super(CPElement, self).__init__(name, CP, state)
        self.group = group

    def _getEnable(self):
        """ getter for enable flag

        :returns: if element is enable
        :rtype: :obj:`bool`
        """
        if self.group and self.name in self.group.keys():
            vl = self.group[self.name]
            if vl is None:
                return False
        return True

    def _getChecked(self):
        """ getter for checked flag

        :returns: if element is checked
        :rtype: :obj:`bool` or `None`
        """
        if not self.group:
            res = self.state.cpgroup
            dsres = self.state.dsgroup
        else:
            res = self.group
            dsres = {}
        if self.name in res.keys():
            return res[self.name] is True
        if self.name in dsres.keys():
            return dsres[self.name] is True
        return False

    def _setChecked(self, status):
        """ setter for checked flag

        :param status: check status
        :type: :obj:`bool` or `None`
        """
        self.state.ddsdirty = True
        if not status:
            dds = self.state.ddsdict
        self.state.ddsdirty = True
        if not self.group:
            ds = self.state.dsgroup
            dc = self.state.cpgroup
        else:
            dc = self.group
            ds = {}
        dc[self.name] = status
        self.state.ddsdirty = True
        if self.name in ds.keys():
            ds[self.name] = status
            self.state.ddsdirty = True
        if not status:
            # res = self.state.cpgroup
            nd = self.state.nodisplay
            if self.name in nd:
                nd.remove(self.name)
        else:
            dds = self.state.ddsdict
        for dd, cp in dds.items():
            if cp == self.name:
                if dd in ds:
                    ds[dd] = status
                    self.state.ddsdirty = True
                if dd in dc:
                    dc[dd] = status
                    self.state.ddsdirty = True
                if not status and dd in nd:
                    nd.remove(dd)

    def _getDisplay(self):
        """ getter for display flag

        :returns: if element should be displayed
        :rtype: :obj:`bool`
        """
        cpres = self.state.cpgroup
        dsres = self.state.dsgroup
        nd = self.state.nodisplay
        acpres = self.state.acpgroup
        mcpres = self.state.mcplist
        if self.name not in nd:
            if (self.name in acpres.keys() and acpres[self.name]) or \
               (self.name in cpres.keys() and cpres[self.name]) or \
               (self.name in cpres.keys() and cpres[self.name]) or \
               (self.name in dsres.keys() and dsres[self.name]) or \
               self.name in mcpres:
                return True
        return False

    def _setDisplay(self, status):
        """ setter for display flag

        :param status: display status
        :type: :obj:`bool` or `None`
        """

        dc = self.state.cpgroup
        mc = self.state.mcplist
        ac = self.state.acpgroup
        ds = self.state.dsgroup
        nd = self.state.nodisplay
        dds = self.state.ddsdict
        found = False
        if self.name in ds.keys():
            found = True
            if self.name in nd:
                if status:
                    nd.remove(self.name)
            else:
                if not status:
                    nd.append(self.name)
                else:
                    ds[self.name] = True
        if self.name in dc.keys():
            found = True
            if self.name in nd:
                if status:
                    nd.remove(self.name)
            else:
                if not status:
                    nd.append(self.name)
                else:
                    dc[self.name] = True
        if self.name in ac.keys():
            found = True
            if self.name in nd:
                if status:
                    nd.remove(self.name)
            else:
                if not status:
                    nd.append(self.name)
                else:
                    ac[self.name] = True
        if self.name in mc:
            found = True
            if self.name in nd:
                if status:
                    nd.remove(self.name)
            else:
                if not status:
                    nd.append(self.name)
        if not found:
            if self.name in nd:
                nd.remove(self.name)
        for dd, cp in dds.items():
            if cp == self.name:
                if dd in nd:
                    if status:
                        nd.remove(dd)
                else:
                    if not status:
                        nd.append(dd)
                    else:
                        if dd in dc.keys():
                            dc[dd] = True
                        ds[dd] = True



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# Element Model

""" device Model """

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from .Element import DS, CP

import json
import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)

#: (:obj:) name=0
NAME = range(1)

#: (:obj:`list` <:obj:`str` > ) synchronization text labels
PROPTEXT = {"synchronization": ["Trigger", "Gate", "Start"]}


class ElementModel(Qt.QAbstractTableModel):
    """ element model
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()
    #: (:class:`taurus.qt.Qt.pyqtSignal`) component checked signal
    componentChecked = Qt.pyqtSignal()
    #: (:class:`taurus.qt.Qt.pyqtSignal`) datachanged  signal
    datachanged = Qt.pyqtSignal(Qt.QModelIndex, Qt.QModelIndex)

    def __init__(self, group=None):
        """ consturctor

        :param group: list of device elements
        :type group: :obj:`list` <:class:`nxsselector.Element.Element`>
        """
        Qt.QAbstractTableModel.__init__(self)
        #: (:obj:`bool`) if enable selection for user
        self.enable = True
        #: (:obj:`bool`) if display enable selection for user
        self.disEnable = True
        #: (:obj:`bool`) if  enable (group widget)
        self.autoEnable = True
        #: (:obj:`list` <:class:`nxsselector.Element.Element`>) \
        #:    list of device elements
        self.group = []
        #: (:obj:`list` <:obj:`str`>) headers
        self.headers = ["Element", "Label", "Display",
                        "Scans", "Contains", "Properties"]

        if group:
            self.group = sorted(group, key=lambda x: x.name, reverse=False)

    def rowCount(self, _=Qt.QModelIndex()):
        """ provides number of model rows

        :returns: number of model rows
        :rtype: :obj:`int`
        """
        return len(self.group)

    def columnCount(self, _=Qt.QModelIndex()):
        """ provides number of model columns

        :returns: number of model columns
        :rtype: :obj:`int`
        """
        return len(self.headers)

    def index(self, row, column, _=Qt.QModelIndex()):
        """ creates a new index from the given parameters

        :param row: element row
        :type row: :obj:`int`
        :param column: element column
        :type column: :obj:`int`
        :returns: created index
        :rtype: :class:`taurus.qt.Qt.QModelIndex`
        """

        return self.createIndex(row, column)

    def __elementCheck(self, device, index):
        """ provides device check status

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :returns: check status
        :rtype: :class:`taurus.qt.Qt.Qt.CheckState`
        """
        if (not (self.flags(index) & Qt.Qt.ItemIsEnabled)
            and self.enable and device.enable) \
           or device.checked:
            return Qt.Qt.Checked
        else:
            return Qt.Qt.Unchecked

    def __displayCheck(self, device, index):
        """ provides device display status

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :returns: display status
        :rtype: :class:`taurus.qt.Qt.Qt.CheckState`
        """
        if (not (self.flags(index) & Qt.Qt.ItemIsEnabled)
            and self.enable and device.enable) \
           or device.checked:
            dds = device.state.ddsdict
            if device.name in dds.keys():
                nd = device.state.nodisplay
                if not dds[device.name] in nd \
                        and dds[device.name]:
                    return Qt.Qt.Checked
            if device.display:
                return Qt.Qt.Checked
            else:
                return Qt.Qt.Unchecked
        else:
            return Qt.Qt.Unchecked

    def __setProperties(self, device, variables):
        """ sets device properties

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :param variables: (name, value) json dictionary with properties
        :type variables: :obj:`str`
        """
        props = device.state.properties
        dname = device.name
        cpvrs = device.state.cpvrdict
        cvars = device.state.configvars
        prs = json.loads(variables)
        for nm, val in prs.items():
            if not nm.startswith("__"):
                if dname in cpvrs.keys() and nm in cpvrs[dname]:
                    if val is not None:
                        cvars[nm] = val
                    elif nm in cvars:
                        cvars.pop(nm)
                else:
                    if nm not in props.keys():
                        props[nm] = {}
                    if val is not None:
                        props[nm][dname] = val
                    elif dname in props[nm].keys():
                        props[nm].pop(dname)
        device.state.setProperties()

    def __properties(self, device):
        """ provides device properties

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :returns: (name, value) json dictionary with properties
        :rtype: :obj:`str`
        """
        cpvrs = device.state.cpvrdict
        cvars = device.state.configvars
        props = device.state.properties
        ochs = device.state.orderedchannels
        chps = device.state.channelprops
        echps = device.state.extrachannelprops
        admindata = device.state.admindata
        tglist = device.state.triggergatelist
        dname = device.name
        contains = dict()
        if dname in cpvrs.keys():
            for vr in cpvrs[dname]:
                if vr not in admindata:
                    contains[vr] = cvars[vr] if vr in cvars.keys() else None

        if dname in ochs:
            for pr in chps:
                if props and pr in props.keys() and \
                   props[pr] and dname in props[pr].keys():
                    contains[pr] = props[pr][dname]
                else:
                    contains[pr] = None
            for pr in echps:
                if props and pr in props.keys() and \
                   props[pr] and dname in props[pr].keys():
                    contains[pr] = props[pr][dname]
                else:
                    contains[pr] = None
        contains["__triggergatelist__"] = tglist
        return json.dumps(contains)

    def __scanSources(self, device):
        """ provides device scan datasources

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :returns: string list of scan datasources separated by spaces
        :rtype: :obj:`str`
        """
        desc = device.state.description
        contains = set()
        for cpg in desc:
            for cp, dss in cpg.items():
                if cp == device.name:
                    if isinstance(dss, dict):
                        for ds, values in dss.items():
                            for vl in values:
                                if len(vl) > 0 and vl[0] == 'STEP':
                                    contains.add(ds)
                                    break
        if contains:
            return " ".join([str(c) for c in sorted(contains)])

    def __descSources(self, device):
        """ provides device description datasources

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :returns: string list of datasources datasources separated by spaces
        :rtype: :obj:`str`
        """
        desc = device.state.description
        contains = set()
        for cpg in desc:
            for cp, dss in cpg.items():
                if cp == device.name:
                    if isinstance(dss, dict):
                        for ds, values in dss.items():
                            for vl in values:
                                if len(vl) == 0 or vl[0] != 'STEP':
                                    contains.add(ds)
                                    break
        if contains:
            return " ".join([str(c) for c in sorted(contains)])

    @classmethod
    def __createList(cls, text, words=7):
        """ creates a comma separated list from the text

        :param text: the given text
        :type device: :obj:`str`
        :returns: comma separated list from the text
        :rtype: :obj:`str`
        """
        lst = str(text).split() if text else []
        cnt = 0
        st = ""
        for sl in lst[:-1]:
            st += sl
            cnt += 1
            if cnt % words:
                st += ', '
            else:
                st += ',\n'

        if len(lst):
            st += lst[-1]
        return st

    def __createTips(self, device, index):
        """ creates device tips

        :param device: device element
        :type device: :class:`nxsselector.Element.Element`
        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :returns: device tips
        :rtype: :obj:`str`
        """
        scans = self.__scanSources(device)
        depends = self.__descSources(device)
        prs = self.__properties(device)
        tscans = self.__createList(scans)
        tdepends = self.__createList(depends)
        text = tscans if tscans else ""
        dds = device.state.ddsdict
        if tdepends:
            text = "%s\n[%s]" % (text, tdepends)

        if device.name in dds:
            sld = dds[device.name]
            if sld:
                text = "%s\n * selected by %s *" % (text, sld)
            else:
                text = "%s\n * timer *" % text

        if prs:
            prs = json.loads(prs)
            tt = " ".join("%s=\"%s\"" % (
                k, (v if k not in PROPTEXT.keys() else PROPTEXT[k][int(v)]))
                for (k, v) in prs.items() if v)
            if tt.strip():
                text = "%s\n(%s)" % (text, tt.strip())

        if text.strip():
            return text

    def data(self, index, role=Qt.Qt.DisplayRole):
        """ provides model data

        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :param role: data model role
        :type role: :class:`taurus.qt.Qt.Qt.ItemDataRole`
        :returns: model data
        :rtype: `str:class:`taurus.qt.Qt.`
        """
        if not index.isValid() or \
                not 0 <= index.row() < len(self.group):
            return
        device = self.group[index.row()]
        column = index.column()
        if column == 0:
            if role == Qt.Qt.DisplayRole:
                return (device.name)
            if role == Qt.Qt.CheckStateRole:
                return self.__elementCheck(device, index)
            if role == Qt.Qt.ToolTipRole:
                tips = self.__createTips(device, index)
                if tips:
                    return (str(tips))
        elif column == 1:
            if role == Qt.Qt.CheckStateRole:
                return
            if device.name in device.state.labels.keys():
                return (device.state.labels[device.name])
        elif column == 2:
            if role == Qt.Qt.CheckStateRole:
                return self.__displayCheck(device, index)
        elif column == 3:
            if role == Qt.Qt.CheckStateRole:
                return
            return (str(self.__scanSources(device)))
        elif column == 4:
            if role == Qt.Qt.CheckStateRole:
                return
            return (str(self.__descSources(device)))
        elif column == 5:
            if role == Qt.Qt.CheckStateRole:
                return
            return (str(self.__properties(device)))
        return ()

    def headerData(self, section, _, role=Qt.Qt.DisplayRole):
        """ provides model data header

        :param section: section number
        :type section: :obj:`int`
        :param role: data model role
        :type role: :class:`taurus.qt.Qt.Qt.ItemDataRole`
        :returns: model data header
        :rtype: `str:class:`taurus.qt.Qt.`
        """
        if role != Qt.Qt.DisplayRole:
            return ()
        if section >= 0 and section < len(self.headers):
            return (self.headers[section])
        return (int(section + 1))

    def flags(self, index):
        """ provides model data flag

        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :returns: model data flag
        :rtype: `str:class:`taurus.qt.Qt.Qt.ItemFlag`
        """
        if not index.isValid():
            return Qt.Qt.ItemIsEnabled

        enable = True
        comp = None
        device = self.group[index.row()]
        enable2 = self.enable and device.enable
        flag = Qt.QAbstractTableModel.flags(self, index)
        column = index.column()
        dds = device.state.ddsdict
        if device.eltype == DS:
            if device.name in dds.keys() and self.autoEnable:
                enable = False
                flag &= ~Qt.Qt.ItemIsEnabled
                comp = dds[device.name]
        elif device.eltype == CP:
            mcp = device.state.mcplist
            # acp = device.state.acplist
            if (self.autoEnable and device.name in mcp) \
               or device.name in dds.keys():
                enable2 = False
                flag &= ~Qt.Qt.ItemIsEnabled
                if device.name in dds.keys():
                    enable = False
                    comp = dds[device.name]
        if column == 0:
            if enable and enable2:
                return Qt.Qt.ItemFlags(flag |
                                       Qt.Qt.ItemIsEnabled |
                                       Qt.Qt.ItemIsUserCheckable)
            else:
                flag &= ~Qt.Qt.ItemIsEnabled
                return Qt.Qt.ItemFlags(
                    flag |
                    Qt.Qt.ItemIsUserCheckable
                )
        elif column == 1:
            flag &= ~Qt.Qt.ItemIsUserCheckable
            if not enable or not enable2:
                flag &= ~Qt.Qt.ItemIsEnabled
            return Qt.Qt.ItemFlags(
                flag |
                Qt.Qt.ItemIsEditable
            )
        if column == 2:
            if not self.disEnable:
                flag &= ~Qt.Qt.ItemIsEnabled
                return Qt.Qt.ItemFlags(flag | Qt.Qt.ItemIsUserCheckable)
            cpncheck = False
            if comp:
                if comp in device.state.mcplist:
                    cpncheck = False
                    if not cpncheck and comp in device.state.nodisplay:
                        cpncheck = True
                elif comp in device.state.acpgroup:
                    cpncheck = not device.state.acpgroup[comp]
                    if not cpncheck and comp in device.state.nodisplay:
                        cpncheck = True
                elif comp in device.state.cpgroup:
                    cpncheck = not device.state.cpgroup[comp]
                    if not cpncheck and comp in device.state.nodisplay:
                        cpncheck = True
                if comp in device.state.dsgroup:
                    cpncheck = not device.state.dsgroup[comp]
                    if not cpncheck and comp in device.state.nodisplay:
                        cpncheck = True
            elif comp is not None:
                cpncheck = True
            if enable or cpncheck:
                return Qt.Qt.ItemFlags(flag |
                                       Qt.Qt.ItemIsEnabled |
                                       Qt.Qt.ItemIsUserCheckable)
            else:
                flag &= ~Qt.Qt.ItemIsEnabled
                return Qt.Qt.ItemFlags(flag | Qt.Qt.ItemIsUserCheckable)
        elif column == 3:
            if enable and enable2:
                return Qt.Qt.ItemFlags(flag | Qt.Qt.ItemIsEnabled)
            else:
                flag &= ~Qt.Qt.ItemIsEnabled
                return Qt.Qt.ItemFlags(flag)
        elif column == 4:
            if enable and enable2:
                return Qt.Qt.ItemFlags(flag | Qt.Qt.ItemIsEnabled)
            else:
                flag &= ~Qt.Qt.ItemIsEnabled
                return Qt.Qt.ItemFlags(flag)
        elif column == 5:
            return Qt.Qt.ItemFlags(flag | Qt.Qt.ItemIsEnabled)

    def setData(self, index, value, role=Qt.Qt.EditRole):
        """ sets model data

        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :param value: element value
        :type value: :class:`taurus.qt.Qt.`
        :param role: data model role
        :type role: :class:`taurus.qt.Qt.Qt.ItemDataRole`
        :returns: if data fas set
        :rtype: :obj:`str`
        """
        if index.isValid() and 0 <= index.row() < len(self.group):
            device = self.group[index.row()]
            column = index.column()
            if column == 0:
                if role == Qt.Qt.CheckStateRole:
                    index3 = self.index(index.row(), 2)
                    device.state.ddsdirty = True
                    device.checked = value
                    self.datachanged.emit(index, index3)
                    self.componentChecked.emit()
                    self.dirty.emit()
                return True
            elif column == 1:
                if role == Qt.Qt.EditRole:
                    label = value.toString()
                    device.state.labels[device.name] = str(label)
                    self.datachanged.emit(index, index)
                    self.dirty.emit()
                    return True
            elif column == 2:
                if role == Qt.Qt.CheckStateRole:
                    index3 = self.index(index.row(), 2)
                    device.display = value
                    self.datachanged.emit(index, index3)
                    self.componentChecked.emit()
                    self.dirty.emit()
                    return True
            elif column == 5:
                if role == Qt.Qt.EditRole:
                    if hasattr(value, "toString"):
                        value = value.toString()
                    self.__setProperties(device, str(value))
                    index5 = self.index(index.row(), 5)
                    self.datachanged.emit(index, index5)
                    self.dirty.emit()
                    return True
        return False


class ElementDelegate(Qt.QStyledItemDelegate):
    """ element delegate
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QStyledItemDelegate.__init__(self, parent)

    def createEditor(self, parent, option, index):
        """ created editor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        :param option: editor option
        :type option: :class:`taurus.qt.Qt.QStyleOptionViewItem`
        :param index: element index
        :type index: :class:`taurus.qt.Qt.QModelIndex`
        :returns: created editor
        :rtype: :class:`taurus.qt.Qt.QWidget`
        """
        if index.column() == 0:
            editor = Qt.QCheckBox(parent)
            return editor
        else:
            return Qt.QItemDelegate.createEditor(self, parent, option, index)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# error message box

""" error message box """

import sys
import logging
import PyTango
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt


class MessageBox(Qt.QObject):
    """ error message box """

    def __init__(self, parent):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QObject.__init__(self, parent)

    @classmethod
    def getText(cls, default, error=None):
        """ provides error message text fro sys.exc_info()

        :param default: default message test
        :type default: :obj:`str`
        :param error: exception to describe
        :type error: :obj:`Exception`
        :returns: exception message
        :rtype: :obj:`str`
        """
        if error is None:
            error = sys.exc_info()[1]
        text = default
        try:
            if isinstance(error, PyTango.DevFailed):
                if hasattr(error, "args"):
                    text = str(
                        "\n".join(["%s " % (err.desc) for err in error.args]))
                else:
                    text = str(
                        "\n".join(["%s " % (err.desc) for err in error]))
            else:
                text = str(error)
        except Exception:
            pass
        return text

    @classmethod
    def warning(cls, parent, title, text, detailedText=None, icon=None):
        """ creates warning messagebox

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        :param title: message box title
        :type title: :obj:`str`
        :param text: message box text
        :type text: :obj:`str`
        :param detailedText: message box detailed text
        :type detailedText: :obj:`str`
        :param icon: message box icon
        :type icon:  :class:`taurus.qt.Qt.QIcon`
        """
        msgBox = Qt.QMessageBox(parent)
        msgBox.setText(title)
        msgBox.setInformativeText(text)
        if detailedText is not None:
            msgBox.setDetailedText(detailedText)
        if icon is None:
            icon = Qt.QMessageBox.Warning
        msgBox.setIcon(icon)
        spacer = Qt.QSpacerItem(800, 0, Qt.QSizePolicy.Minimum,
                                Qt.QSizePolicy.Expanding)
        layout = msgBox.layout()
        layout.addItem(spacer, layout.rowCount(), 0, 1, layout.columnCount())
        msgBox.exec_()
        msgBox.setParent(None)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# module with different table views

""" component element table views """


try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

import logging
import json

from .DynamicTools import DynamicTools
from .LDataDlg import LDataDlg
from .LDataDlg import LExDataDlg

#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


class RightClickCheckBox(Qt.QCheckBox):
    """ right click checkbox table widget
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) rightClick signal
    rightClick = Qt.pyqtSignal()

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QCheckBox.__init__(self, parent)

    @Qt.pyqtSlot()
    def mousePressEvent(self, event):
        """ excecutes right click action

        :param event: mouse event
        :type event: :class:`taurus.qt.Qt.QEvent`
        """
        Qt.QCheckBox.mousePressEvent(self, event)
        if event.button() == Qt.Qt.RightButton:
            self.rightClick.emit()


class CheckerLabelWidget(Qt.QWidget):
    """ checkbox table widget
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QWidget.__init__(self, parent)
        #: (:class:`taurus.qt.Qt.QCheckBox`) element checkbox widget
        self.checkBox = Qt.QCheckBox(self)
        #: (:class:`taurus.qt.Qt.QCheckBox`) element label widget
        self.label = Qt.QLabel(self)
        layout = Qt.QHBoxLayout()
        layout.addWidget(self.checkBox)
        layout.addWidget(self.label)
        layout.setContentsMargins(2, 2, 2, 2)
        self.setLayout(layout)

    def isChecked(self):
        """ provides checkbox status

        :returns: check status
        :rtype: :obj:`bool`
        """
        return self.checkBox.isChecked()

    def setChecked(self, status):
        """ sets checkbox status

        :param status: check status
        :type status: :obj:`bool`
        """
        self.label.setEnabled(status)
        self.checkBox.setChecked(status)

    def setCheckable(self, status):
        """ sets ckeckable checkbox flag

        :param status: checkable flag
        :type status: :obj:`bool`
        """
        return self.checkBox.setCheckable(status)

    def setEnabled(self, status):
        """ sets enable checkbox flag

        :param status: enable flag
        :type status: :obj:`bool`
        """
        return self.checkBox.setEnabled(status)

    def setSizePolicy(self, policy):
        """ sets label size policy

        :param policy: size policy
        :type policy: :class:`taurus.qt.Qt.QSizePolicy`
        """
        self.label.setSizePolicy(policy)
        Qt.QWidget.setSizePolicy(self, policy)

    def setText(self, text):
        """ sets label text

        :param policy: label text
        :type policy: :obj:`str` or :obj:`str`
        """
        self.label.setText(text)


class TableView(Qt.QTableView):
    """ table view with streached last column
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """

        Qt.QTableView.__init__(self, parent)
        self.verticalHeader().setVisible(False)
#        self.horizontalHeader().setVisible(False)
        self.horizontalHeader().setStretchLastSection(True)
        try:
            self.horizontalHeader().setResizeMode(Qt.QHeaderView.Stretch)
        except Exception:
            # workaround for bug in pyqt
            self.horizontalHeader().setSectionResizeMode(
                Qt.QHeaderView.Stretch)


class OneTableView(Qt.QTableView):
    """ table view with stretched last column un unvisible horizontal header
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QTableView.__init__(self, parent)
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().setVisible(False)
        self.horizontalHeader().setStretchLastSection(True)
        try:
            self.horizontalHeader().setResizeMode(Qt.QHeaderView.Stretch)
        except Exception:
            # workaround for bug in pyqt
            self.horizontalHeader().setSectionResizeMode(
                Qt.QHeaderView.Stretch)

    def reset(self):
        """ resets table view and hides view columns
        """
        Qt.QTableView.reset(self)
        for i in range(1, 6):
            self.hideColumn(i)


class CheckerView(Qt.QWidget):
    """ element view with redefined checkboxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QWidget.__init__(self, parent)
        #: (:class:`taurus.qt.Qt.QAbstractTableModel`) view model
        self.model = None
        #: (:class:`taurus.qt.Qt.QGridLayout`) main grid layout
        self.glayout = Qt.QGridLayout(self)
        #: (:obj:`list` <:class:`taurus.qt.Qt.QGridLayout`> ) element widgets
        self.widgets = []
        #: (:class:`taurus.qt.Qt.QSignalMapper`) widget check status mapper
        self.mapper = Qt.QSignalMapper(self)
        #: (:class:`taurus.qt.Qt.QSignalMapper`) widget property mapper
        self.pmapper = None
        #: (:class:`taurus.qt.Qt.QSignalMapper`) widget display status mapper
        self.dmapper = None
        #: (:obj:`list` <:class:`taurus.qt.Qt.QGridLayout`> ) \
        #:     display checkbox widgets
        self.displays = []
        self.mapper.mapped.connect(self.checked)
        #: (:class:`taurus.qt.Qt.QSpacerItem`) spacer view item
        self.spacer = None
        #: (:obj:`type`) widget type
        self.widget = Qt.QCheckBox
        #: (:obj:`bool`) if widget are centered
        self.center = True
        #: (:obj:`int`) maximal number of rows in the column
        self.rowMax = 0
        #: (:obj:`int`) font size for in component column view
        self.fontSize = 11
        #: (:obj:`int`) selected widget row (widget id)
        self.selectedWidgetRow = None
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = True
        #: (:obj:`bool`) if names should be shown
        self.showNames = True
        self.setContextMenuPolicy(Qt.Qt.PreventContextMenu)

    def close(self):
        """ widget close method which disconnect signals """

        self.mapper.mapped.disconnect(self.checked)
        if self.model:
            self.model.dataChanged.disconnect(self.reset)
            self.model.modelReset.disconnect(self.reset)
        Qt.QWidget.close(self)

    @Qt.pyqtSlot(int)
    def checked(self, row):
        """ updates the element model check status for the given widget row

        :param row: widget row
        :type row: :obj:`int`
        """
        self.selectedWidgetRow = row
        ind = self.model.index(row, 0)
        value = (self.widgets[row].isChecked())
        if self.dmapper:
            if not value:
                self.displays[row].setChecked(bool(False))

        self.model.setData(ind, value, Qt.Qt.CheckStateRole)

    def setModel(self, model):
        """
        set the current view model

        :param model: view model to be set
        :type model: :class:`taurus.qt.Qt.QAbstractTableModel`

        """
        self.model = model
        self.model.dataChanged.connect(self.reset)
        self.model.modelReset.connect(self.reset)
        self.reset()

    def clearLayout(self):
        """ clears layouts of the view
        """

        if self.glayout:
            self.widgets = []
            if self.dmapper:
                self.displays = []
            self.spacer = None
            DynamicTools.cleanupLayoutWithItems(self.glayout)
            self.glayout = Qt.QGridLayout(self)

    def connectMapper(self):
        """ reconnects mapper
        """
        if self.mapper:
            self.mapper.mapped.disconnect(self.checked)
        self.mapper = Qt.QSignalMapper(self)
        self.mapper.mapped.connect(self.checked)

    @Qt.pyqtSlot()
    @Qt.pyqtSlot(Qt.QModelIndex, Qt.QModelIndex)
    def reset(self):
        """ resets the view
        """
        self.hide()
        self.clearLayout()
        self.connectMapper()
        self.updateState()
        if self.selectedWidgetRow is not None:
            if len(self.widgets) > self.selectedWidgetRow:
                self.widgets[self.selectedWidgetRow].setFocus()
            self.selectedWidgetRow = None
        self.show()

    def __findRowNumber(self, rowMax, rowCount):
        """ finds physical row number for the element widget

        :param rowMax: maximal number of rows in the column
        :type rowMax: :obj:`int`
        :param rowCount: current model widget row (widget id)
        :type rowCount: :obj:`int`
        """
        rowNo = rowMax
        if rowNo < 1:
            rowNo = 1
        fullColumnNo = rowCount // rowNo
        lastRowNo = rowCount % rowNo
        while lastRowNo and fullColumnNo < rowNo - lastRowNo:
            rowNo -= 1
            lastRowNo += fullColumnNo
        return rowNo

    def updateState(self):
        """ updates state of the view
        """
        if self.model is not None:
            rowCount = self.model.rowCount()
            rowNo = self.__findRowNumber(self.rowMax, rowCount)

            for row in range(rowCount):

                cb, ds = self.__setWidgets(row)
                self.__setNameTips(row, cb)
                self.__createGrid(row, cb, ds, rowNo)

            if not self.spacer:
                self.spacer = Qt.QSpacerItem(
                    10, 10,
                    Qt.QSizePolicy.Minimum,
                    Qt.QSizePolicy.Expanding)
                self.glayout.addItem(self.spacer)
        self.__resetStretchFactors()
        self.update()
        self.updateGeometry()

    def __resetStretchFactors(self):
        """ resets stretch factors """
        grp = self.parent()
        if hasattr(grp, "parent"):
            frm = grp.parent()
            stretch = 0
            grps = [(gp, None) for gp in frm.findChildren(Qt.QGroupBox)]
            hfrmlayout = frm.layout()
            vfrmlayouts = [hl for hl in hfrmlayout.children()
                           if isinstance(hl, Qt.QVBoxLayout)]
            gvls = []
            if vfrmlayouts:
                for vl in vfrmlayouts:
                    kids = [
                        (vl.itemAt(ind).widget(), vl)
                        for ind in range(vl.count())
                        if isinstance(vl.itemAt(ind).widget(), Qt.QGroupBox)]
                    gvls.append(kids)
            else:
                gvls = [grps]

            for gvl in gvls:
                vst = 0
                lvl = None
                for gr, vl in gvl:
                    gst = 0
                    lvl = vl
                    mvws = gr.findChildren(CheckerView)
                    for mvw in mvws:
                        if hasattr(mvw.model, 'rowCount'):
                            lgst = mvw.model.rowCount() // self.rowMax \
                                + (1 if (mvw.model.rowCount() % self.rowMax)
                                   else 0)
                            if lgst > gst:
                                gst = lgst
                    vst = gst
                if lvl:
                    hfrmlayout.setStretchFactor(lvl, vst)
                stretch += vst
            if hasattr(frm, "parent"):
                lyt = frm.parent().layout()
                sizestt = [0, 1, 2, 3, 3, 4, 4, 4, 5, 5, 5, 5]
                if stretch >= len(sizestt):
                    nstretch = 6
                elif stretch < 0:
                    nstretch = 0
                else:
                    nstretch = sizestt[stretch]
                lyt.setStretchFactor(frm, nstretch)

    def __createGrid(self, row, cb, ds, rowNo=None):
        """ add widget into the view grid

        :param row: the row number
        :type row: :obj:`int`
        :param cb: status checkbox or button
        :type cb: :class:`taurus.qt.Qt.QWidget`
        :param ds: display checkbox or button
        :type ds: :class:`taurus.qt.Qt.QWidget`
        :param rowNo: the number of row in the group
        :type rowNo: :obj:`int`
        """
        rowNo = self.rowMax if not rowNo else rowNo
        ind = self.model.index(row, 0)
        ind2 = self.model.index(row, 2)
        status = self.model.data(ind, role=Qt.Qt.CheckStateRole)
        dstatus = self.model.data(ind2, role=Qt.Qt.CheckStateRole)
        if status is not None:
            cb.setChecked(bool(status))
        if self.dmapper:
            if dstatus is not None:
                ds.setChecked(bool(dstatus))
        if row >= len(self.widgets):
            if rowNo:
                lrow = row % rowNo
                lcol = row // rowNo
            else:
                lrow = row
                lcol = 0
            if self.dmapper:
                lrow = lrow + 1
                lcol = 2 * lcol

                if lrow == 1:
                    self.glayout.addWidget(
                        Qt.QLabel(self.slabel), 0, lcol, 1, 1)
                    self.glayout.addWidget(
                        Qt.QLabel(self.dlabel), 0, lcol + 1, 1, 1,
                        Qt.Qt.AlignCenter)
                self.glayout.addWidget(ds, lrow, lcol + 1, 1, 1,
                                       Qt.Qt.AlignCenter)
            self.glayout.addWidget(cb, lrow, lcol, 1, 1)
            self.widgets.append(cb)

            if hasattr(cb, "clicked"):
                cb.clicked.connect(self.mapper.map)
            self.mapper.setMapping(cb, self.widgets.index(cb))
            if self.dmapper:
                self.displays.append(ds)
                if hasattr(ds, "clicked"):
                    ds.clicked.connect(self.dmapper.map)
                self.dmapper.setMapping(ds, self.displays.index(ds))
            if self.pmapper:
                if hasattr(cb, "rightClick"):
                    cb.rightClick.connect(self.pmapper.map)
                self.pmapper.setMapping(cb, self.widgets.index(cb))

    def __setWidgets(self, row):
        """ creates element widgets

        :param row: the row number
        :type row: :obj:`int`
        :returns: (checkbox widget, display widget)
        :rtype: (:class:`taurus.qt.Qt.QWidget`, :class:`taurus.qt.Qt.QWidget`)
        """
        ind = self.model.index(row, 0)
        flags = self.model.flags(ind)
        ind2 = self.model.index(row, 2)
        flags2 = self.model.flags(ind2)
        ds = None
        cb = None
        if row < len(self.widgets):
            cb = self.widgets[row]
            if self.dmapper:
                ds = self.displays[row]
        else:
            cb = self.widget()
            font = cb.font()
            font.setPointSize(self.fontSize)
            cb.setFont(font)
            if hasattr(cb, "setCheckable"):
                cb.setCheckable(True)
            if self.dmapper:
                ds = self.widget()
                font = ds.font()
                font.setPointSize(self.fontSize)
                ds.setFont(font)
                if hasattr(ds, "setCheckable"):
                    ds.setCheckable(True)
            if hasattr(cb, "setSizePolicy") and self.center:
                sizePolicy = Qt.QSizePolicy(
                    Qt.QSizePolicy.Fixed, Qt.QSizePolicy.Fixed)
                sizePolicy.setHorizontalStretch(10)
                sizePolicy.setVerticalStretch(0)
                sizePolicy.setHeightForWidth(
                    cb.sizePolicy().hasHeightForWidth())
                cb.setSizePolicy(sizePolicy)

        cb.setEnabled(bool(Qt.Qt.ItemIsEnabled & flags))
        if self.dmapper:
            ds.setEnabled(bool(Qt.Qt.ItemIsEnabled & flags2))
        return (cb, ds)

    def __setNameTips(self, row, cb):
        """ set name tips of element widget

        :param row: the row number
        :type row: :obj:`int`
        :param cb: checkbox widget
        :type cb: :class:`taurus.qt.Qt.QWidget`
        """
        ind = self.model.index(row, 0)
        name = self.model.data(ind, role=Qt.Qt.DisplayRole)
        ind1 = self.model.index(row, 1)
        label = self.model.data(ind1, role=Qt.Qt.DisplayRole)

        if name:
            if self.showLabels and label and \
                    str(label).strip():
                if self.showNames:
                    cb.setText("%s [%s]" % (
                        str(label),
                        str(name)))
                else:
                    cb.setText("%s" % (str(label)))
            else:
                cb.setText(str(name))

            # works with:
            # QT_API=pyqt4 ./nxselector  -t cleanlooks
            # QT_API=pyqt5 ./nxselector
            cb.setStyleSheet(
                "QCheckBox:unchecked{ color: black; }"
                "QCheckBox:checked{ color: blue; }"
                "QCheckBox:disabled{ color: gray; }"
            )
        text = self.model.data(ind, role=Qt.Qt.ToolTipRole)
        text = str(text) if text else ""
        if self.showLabels:
            if self.showNames:
                if text.strip():
                    cb.setToolTip(text)
            else:
                if text.strip():
                    cb.setToolTip(
                        "%s: %s" % (str(name), text))
                else:
                    cb.setToolTip(str(name))
        else:
            ln = str(label) if label else str(name)
            if text.strip():
                cb.setToolTip("%s: %s" % (ln, text))
            else:
                cb.setToolTip(ln)


class CheckDisView(CheckerView):
    """ element view with redefined checkboxes with display boxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:class:`taurus.qt.Qt.QSignalMapper`) widget display status mapper
        self.dmapper = Qt.QSignalMapper(self)
        self.dmapper.mapped.connect(self.dchecked)
        #: (:obj:`bool`) if widget are centered
        self.center = False
        #: (:obj:`str`) display status label text
        self.dlabel = 'Dis.'
        #: (:obj:`str`) check status label text
        self.slabel = 'Sel.'

    @Qt.pyqtSlot(int)
    def dchecked(self, row):
        """ updates the element model display for the given widget row

        :param row: widget row
        :type row: :obj:`int`
        """
        self.selectedWidgetRow = row
        ind = self.model.index(row, 2)
        value = (self.displays[row].isChecked())
        self.model.setData(ind, value, Qt.Qt.CheckStateRole)

    def connectMapper(self):
        """ reconnects mapper
        """
        CheckerView.connectMapper(self)
        if self.dmapper:
            self.dmapper.mapped.disconnect(self.dchecked)
        self.dmapper = Qt.QSignalMapper(self)
        self.dmapper.mapped.connect(self.dchecked)

    def close(self):
        """ widget close method which disconnect signals """
        self.dmapper.mapped.disconnect(self.dchecked)
        CheckerView.close(self)


class CheckPropView(CheckDisView):
    """ element view with property widget
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckDisView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = RightClickCheckBox
        #: (:class:`taurus.qt.Qt.QSignalMapper`) widget property mapper
        self.pmapper = Qt.QSignalMapper(self)
        self.pmapper.mapped.connect(self.pchecked)
        self.propdlg = LDataDlg

    @Qt.pyqtSlot(int)
    def pchecked(self, row):
        """ updates the element model properties for the given widget row

        :param row: widget row
        :type row: :obj:`int`
        """
        self.selectedWidgetRow = row
        ind = self.model.index(row, 0)
        name = self.model.data(ind, role=Qt.Qt.DisplayRole)
        ind5 = self.model.index(row, 5)
        props = self.model.data(ind5, role=Qt.Qt.DisplayRole)
        if props:
            prs = json.loads(str(props))
            dform = self.propdlg(self)
            dform.label = name
            if "__triggergatelist__" in prs:
                dform.synchronizers = list(prs["__triggergatelist__"])
            dform.dtype = prs["data_type"] if "data_type" in prs else None
            dform.shape = prs["shape"] if "shape" in prs else None
            dform.link = prs["link"] if "link" in prs else None
            dform.path = prs["nexus_path"] if "nexus_path" in prs else None
            dform.canfail = prs["canfail"] if "canfail" in prs else None
            dform.addVariables(prs)
            dform.createGUI()

            dform.ui.labelLineEdit.setEnabled(False)
            if dform.exec_():
                if "data_type" in prs:
                    prs["data_type"] = dform.dtype or None
                    prs["link"] = dform.link
                    prs["canfail"] = dform.canfail
                    prs["shape"] = dform.shape
                    prs["nexus_path"] = dform.shape or None
                for nm, val in dform.variables.items():
                    prs[nm] = val
                self.model.setData(ind5, (
                    str(str(json.dumps(prs)))))

    def connectMapper(self):
        """ reconnects mappers
        """
        CheckDisView.connectMapper(self)
        if self.pmapper:
            self.pmapper.mapped.disconnect(self.pchecked)
        self.pmapper = Qt.QSignalMapper(self)
        self.pmapper.mapped.connect(self.pchecked)

    def close(self):
        """ widget close method which disconnect signals """
        self.pmapper.mapped.disconnect(self.pchecked)
        CheckDisView.close(self)


class CheckExPropView(CheckPropView):
    """ element view with extended property widget
    """

    def __init__(self, parent=None):
        CheckPropView.__init__(self, parent)
        #: (:obj:`bool`) if widget are centered
        self.propdlg = LExDataDlg


class RadioView(CheckerView):
    """ element view with radio checkboxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = Qt.QRadioButton


class LeftRadioView(CheckerView):
    """ element view with left radio checkboxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = Qt.QRadioButton
        #: (:obj:`bool`) if widget are centered
        self.center = False


class LeftCheckerView(CheckerView):
    """ element view with left checkboxes
    """

    def __init__(self, parent=None):
        CheckerView.__init__(self, parent)
        #: (:obj:`bool`) if widget are centered
        self.center = False


class ButtonView(CheckerView):
    """ element view with button checkboxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = Qt.QPushButton
        #: (:obj:`bool`) if widget are centered
        self.center = False


class CheckerViewNL(CheckerView):
    """ element view with checkboxes without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class LeftCheckerViewNL(LeftCheckerView):
    """ element view with left checkboxes without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        LeftCheckerView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class ButtonViewNL(ButtonView):
    """ element view with button checkboxes without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        ButtonView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class RadioViewNL(RadioView):
    """ element view with radio checkboxes without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        RadioView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class LeftRadioViewNL(LeftRadioView):
    """ element view with left radio checkboxes without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        LeftRadioView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class CheckerViewNN(CheckerView):
    """ element view with checkboxes with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class LeftCheckerViewNN(LeftCheckerView):
    """ element view with checkboxes with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        LeftCheckerView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class ButtonViewNN(ButtonView):
    """ element view with button checkboxes with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        ButtonView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class RadioViewNN(RadioView):
    """ element view with radio checkboxes with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        RadioView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class LeftRadioViewNN(LeftRadioView):
    """ element view with left radio checkboxes with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        LeftRadioView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class RadioDisView(CheckDisView):
    """ element view with left radio checkboxes and display boxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckDisView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = Qt.QRadioButton


class ButtonDisView(CheckDisView):
    """ element view with left button checkboxes and display boxes
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckDisView.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = Qt.QPushButton
        #: (:obj:`bool`) if widget are centered
        self.center = False


class CheckDisViewNL(CheckDisView):
    """ element view with left button checkboxes and display boxes
    without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckDisView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class CheckPropViewNL(CheckPropView):
    """ element view with checkboxes and properties
    without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckPropView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class CheckExPropViewNL(CheckExPropView):
    """ element view with checkboxes and properties
    without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckExPropView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class ButtonDisViewNL(ButtonDisView):
    """ element view with left button checkboxes and display boxes
    without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        ButtonDisView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class RadioDisViewNL(RadioDisView):
    """ element view with left radio checkboxes and display boxes
    without name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        RadioDisView.__init__(self, parent)
        #: (:obj:`bool`) if name labels should be shown
        self.showLabels = False


class CheckDisViewNN(CheckDisView):
    """ element view with redefined checkboxes with display boxes
    without names
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckDisView.__init__(self, parent)
        self.showNames = False


class CheckPropViewNN(CheckPropView):
    """ element view with redefined checkboxes with properties
    without names
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckPropView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class CheckExPropViewNN(CheckExPropView):
    """ element view with redefined checkboxes with properties
    without names
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckExPropView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class CheckerLabelViewNN(CheckerViewNN):
    """ element view with CheckerLabelWidget checkboxes with only name labels
    """
    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        CheckerViewNN.__init__(self, parent)
        #: (:obj:`type`) widget type
        self.widget = CheckerLabelWidget


class ButtonDisViewNN(ButtonDisView):
    """ element view with button checkboxes and display boxes
    with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        ButtonDisView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False


class RadioDisViewNN(RadioDisView):
    """ element view with radio checkboxes and display boxes
    with only name labels
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        RadioDisView.__init__(self, parent)
        #: (:obj:`bool`) if names should be shown
        self.showNames = False



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# storage tab

""" storage tab """

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt
from .EdListDlg import EdListDlg
from .GroupsDlg import GroupsDlg
from .InfoDlg import InfoDlg
from .OrderDlg import OrderDlg
from .PropertiesDlg import PropertiesDlg
from .MessageBox import MessageBox

import logging
import json
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


class Storage(Qt.QObject):
    """ storage tab """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()
    #: (:class:`taurus.qt.Qt.pyqtSignal`) resetviews signal
    resetViews = Qt.pyqtSignal()
    #: (:class:`taurus.qt.Qt.pyqtSignal`) resetall signal
    resetAll = Qt.pyqtSignal()
    #: (:class:`taurus.qt.Qt.pyqtSignal`) updategroups signal
    updateGroups = Qt.pyqtSignal()

    def __init__(self, ui, state=None, simplemode=False):
        """ constructor

        :param ui: ui instance
        :type ui: :class:`taurus.qt.qtgui.util.ui.__UI`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param simpleMode: if simple display mode
        :type simpleMode: :obj:`bool`
        """
        Qt.QObject.__init__(self)

        #: (:class:`taurus.qt.qtgui.util.ui.__UI`) ui instance
        self.ui = ui
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = state
        #: (:class:`taurus.qt.Qt.QHBoxLayout`) the main tab layout
        self.__layout = None
        #: (:obj:`list` <:class:`taurus.qt.Qt.QHBoxLayout`>) timer widget list
        self.__tWidgets = []
        #: (:obj:`bool`) only selected flag
        self.__onlyselected = False
        #: (:obj:`bool`) simple mode
        self.__simplemode = simplemode
        #: (:obj:`str`) module label
        self.__moduleLabel = 'module'
        self.connectSignals()
        #: (:obj:`bool`) connected flag
        self.__connected = True

    def connectTimerButtons(self):
        """ connects timer signals
        """
        self.ui.timerDelPushButton.clicked.connect(self.__delTimer)
        self.ui.timerAddPushButton.clicked.connect(self.__addTimer)

    def disconnectSignals(self):
        """ disconnects all tab signals
        """
        logger.debug("disconnect signals")
        if self.__connected:
            self.__connected = False
            self.ui.fileScanDirToolButton.pressed.disconnect(self.__setDir)
            self.ui.fileScanDirLineEdit.editingFinished.disconnect(
                self.__dirChanged)
            self.ui.fileScanLineEdit.editingFinished.disconnect(
                self.__fileChanged)
            self.ui.fileScanDirLineEdit.textEdited.disconnect(
                self.__dirty)
            self.ui.fileScanLineEdit.textEdited.disconnect(
                self.__dirty)
            # measurement group

            self.ui.mntTimerComboBox.currentIndexChanged.disconnect(self.apply)
            for cb in self.__tWidgets:
                cb.currentIndexChanged.disconnect(self.apply)

            if hasattr(self.ui, "timerDelPushButton"):
                self.ui.timerDelPushButton.clicked.disconnect(self.__delTimer)
                self.ui.timerAddPushButton.clicked.disconnect(self.__addTimer)
            self.ui.mntGrpToolButton.pressed.disconnect(self.__mntgrp_deleted)
            self.ui.mntGrpComboBox.currentIndexChanged.disconnect(
                self.__mntgrp_changed)
            self.ui.mntGrpComboBox.lineEdit().editingFinished.disconnect(
                self.__mntgrp_edited)
            self.ui.mntServerLineEdit.editingFinished.disconnect(self.apply)
            self.ui.mntGrpComboBox.lineEdit().textEdited.disconnect(
                self.__dirty)
            self.ui.mntServerLineEdit.textEdited.disconnect(self.__dirty)

            # device group
            self.ui.devWriterLineEdit.editingFinished.disconnect(self.apply)
            self.ui.devConfigLineEdit.editingFinished.disconnect(self.apply)
            self.ui.devWriterLineEdit.textEdited.disconnect(self.__dirty)
            self.ui.devConfigLineEdit.textEdited.disconnect(self.__dirty)

            # dynamic component group
            self.ui.dcLinksCheckBox.clicked.disconnect(self.apply)
            self.ui.dcPathLineEdit.editingFinished.disconnect(self.apply)
            self.ui.dcPathLineEdit.textEdited.disconnect(self.__dirty)

            # others group
            self.ui.othersEntryCheckBox.clicked.disconnect(self.apply)
            self.ui.devConfigPushButton.clicked.disconnect(self.__variables)
            self.ui.propPushButton.clicked.disconnect(self.__props)
            self.ui.labelsPushButton.clicked.disconnect(self.__labels)
            self.ui.orderToolButton.clicked.disconnect(self.__order)

            self.ui.groupsPushButton.clicked.disconnect(self.__groups)
            self.ui.errorsPushButton.clicked.disconnect(self.__errors)
            self.ui.infoPushButton.clicked.disconnect(self.__info)
            logger.debug("disconnect signals END")

    def connectSignals(self):
        """ connects all tab signals
        """
        logger.debug("connect signals")
        self.__connected = True
        self.ui.fileScanDirToolButton.pressed.connect(self.__setDir)
        self.ui.fileScanDirLineEdit.editingFinished.connect(self.__dirChanged)
        self.ui.fileScanLineEdit.editingFinished.connect(self.__fileChanged)
        self.ui.fileScanDirLineEdit.textEdited.connect(self.__dirty)
        self.ui.fileScanLineEdit.textEdited.connect(self.__dirty)
        # measurement group

        self.ui.mntTimerComboBox.currentIndexChanged.connect(self.apply)
        for cb in self.__tWidgets:
            cb.currentIndexChanged.connect(self.apply)

        if hasattr(self.ui, "timerDelPushButton"):
            self.ui.timerDelPushButton.clicked.connect(self.__delTimer)
            self.ui.timerAddPushButton.clicked.connect(self.__addTimer)
        self.ui.mntGrpToolButton.pressed.connect(self.__mntgrp_deleted)
        self.ui.mntGrpComboBox.currentIndexChanged.connect(
            self.__mntgrp_changed)
        self.ui.mntGrpComboBox.lineEdit().editingFinished.connect(
            self.__mntgrp_edited)
        self.ui.mntServerLineEdit.editingFinished.connect(self.apply)
        self.ui.mntGrpComboBox.lineEdit().textEdited.connect(
            self.__dirty)
        self.ui.mntServerLineEdit.textEdited.connect(self.__dirty)

        # device group
        self.ui.devWriterLineEdit.editingFinished.connect(self.apply)
        self.ui.devConfigLineEdit.editingFinished.connect(self.apply)
        self.ui.devWriterLineEdit.textEdited.connect(self.__dirty)
        self.ui.devConfigLineEdit.textEdited.connect(self.__dirty)

        # dynamic component group
        self.ui.dcLinksCheckBox.clicked.connect(self.apply)
        self.ui.dcPathLineEdit.editingFinished.connect(self.apply)
        self.ui.dcPathLineEdit.textEdited.connect(self.__dirty)

        # others group
        self.ui.othersEntryCheckBox.clicked.connect(self.apply)
        self.ui.devConfigPushButton.clicked.connect(self.__variables)
        self.ui.propPushButton.clicked.connect(self.__props)
        self.ui.labelsPushButton.clicked.connect(self.__labels)
        self.ui.orderToolButton.clicked.connect(self.__order)

        self.ui.groupsPushButton.clicked.connect(self.__groups)
        self.ui.errorsPushButton.clicked.connect(self.__errors)
        self.ui.infoPushButton.clicked.connect(self.__info)
        logger.debug("connect signals END")

    def updateMntGrpComboBox(self):
        """ updates a value of measurement group combo box
        """
        self.disconnectSignals()
        try:
            self.ui.mntGrpComboBox.clear()
            for mg in self.state.avmglist:
                self.ui.mntGrpComboBox.addItem(mg)
            if self.state.mntgrp not in self.state.avmglist:
                self.ui.mntGrpComboBox.addItem(self.state.mntgrp)
            ind = self.ui.mntGrpComboBox.findText(self.state.mntgrp)
            self.ui.mntGrpComboBox.setCurrentIndex(ind)
        finally:
            self.connectSignals()

    @Qt.pyqtSlot()
    def __variables(self):
        """ changes configuration variables
        """
        dform = EdListDlg(self.ui.storage)
        dform.widget.record = self.state.configvars
        dform.simple = True
        dform.available_names = self.state.vrcpdict.keys()
        dform.createGUI()
        dform.exec_()
        if dform.dirty:
            self.resetViews.emit()
            self.dirty.emit()

    @Qt.pyqtSlot()
    def __labels(self):
        """ changes component labels
        """
        dform = EdListDlg(self.ui.storage)
        dform.widget.record = self.state.labels
        dform.simple = True
        dform.headers = ["Element", "Label"]
        dform.available_names = list(
            set(self.state.avcplist) | set(self.state.avdslist))

        dform.createGUI()
        dform.exec_()
        if dform.dirty:
            self.resetViews.emit()

    @Qt.pyqtSlot()
    def __order(self):
        """ changes the channel order
        """
        dform = OrderDlg(self.ui.storage)
        dform.channels = list(self.state.orderedchannels)
        dform.selected = list(
            set([cp for cp in self.state.cpgroup.keys()
                 if self.state.cpgroup[cp]])
            | set([ds for ds in self.state.dsgroup.keys()
                   if self.state.dsgroup[ds]])
            | set(self.state.timers or []))
        dform.onlyselected = self.__onlyselected

        dform.createGUI()
        dform.exec_()
        self.__onlyselected = dform.onlyselected
        if dform.dirty:
            self.state.orderedchannels = list(dform.channels)
            self.dirty.emit()

    @Qt.pyqtSlot()
    def __info(self):
        """ shows the server info
        """
        dform = InfoDlg(self.ui.storage)
        dform.state = self.state
        dform.createGUI()
        dform.exec_()

    def showErrors(self, errors=None):
        """ shows errors

        :param errors: error list
        :type errors: :obj:`list` < :obj:`str` >
        """
        if errors is None:
            errors = self.state.fetchErrors()
        text = ""
        details = ""
        comps = ""
        for er in errors:
            try:
                jer = json.loads(er)
                if comps:
                    comps += ", "
                comps += "'" + str(jer["component"]) + "'"
                ler = "'" + str(jer["component"]) + "' because of '" + \
                    str(jer["datasource"]) + "'\n"
                der = "" + str(jer["component"]) + "(" + \
                    str(jer["datasource"]) + "):\n" \
                    + str(jer["message"]) + "\n"
            except Exception:
                ler = str(er) + "\n"
                der = ler
            text += ler
            details += der
        if errors:
            MessageBox.warning(
                self.ui.storage,
                "NXSelector: Descriptive Component(s): "
                "%s will not be stored" % comps,
                str(text), "%s" % str(details))

    @Qt.pyqtSlot()
    def __errors(self):
        """ shows error dialog """
        errors = self.state.fetchErrors()
        self.showErrors(errors)
        if not errors:
            Qt.QMessageBox.information(
                self.ui.storage,
                "NXSelector: Descrption Component:",
                "Tango Servers of Description Components are ON")

    @Qt.pyqtSlot()
    def __groups(self):
        """ runs detgroups or descgroups depending on tab index"""
        index = self.ui.tabWidget.currentIndex()
        if index == 0:
            self.__detgroups()
        elif index == 1:
            self.__descgroups()

    def __detgroups(self):
        """ changes detector component groups
        """
        dform = GroupsDlg(self.ui.storage)
        dform.state = self.state
        # DAC  to be hidden via reselector property
        hidden = set(self.state.mcplist)
        hidden.update(self.state.mutedChannels)
        hidden.update(set(self.state.orderedchannels))
        hidden.update(set(self.state.acqchannels)
                      - set(self.state.motors)
                      - set(self.state.ioregisters))
        hidden.update([cp for cp in self.state.acpgroup.keys()
                       if self.state.acpgroup[cp]])

        stcomps = self.state.stepComponents()
        dform.components = dict(
            (cp, False) for cp in stcomps
            if cp not in hidden and not cp.startswith("__"))
        dform.components.update(
            dict((cp, True) for cp in self.state.cpgroup.keys()
                 if (cp not in hidden and not cp.startswith("__"))))

        cldsources = self.state.clientDataSources()

        hidden.update(
            set(cldsources)
            - set(self.state.motors)
            - set(self.state.ioregisters))
        dform.datasources = dict(
            (cp, False) for cp in self.state.avdslist
            if cp not in hidden and not cp.startswith("__"))
        dform.datasources.update(
            dict((cp, True) for cp in self.state.dsgroup.keys()
                 if (cp not in hidden and not cp.startswith("__"))))

        dform.createGUI()
        dform.exec_()
        if dform.dirty:
            if dform.newdatasources:
                self.state.createDataSources(dform.newdatasources)
            self.__updateGroup(self.state.cpgroup, dform.components, False)
            self.__updateGroup(self.state.dsgroup, dform.datasources, False)
            self.resetViews.emit()
            self.dirty.emit()

    def __descgroups(self):
        """ changes descriptive component groups
        """
        dform = GroupsDlg(self.ui.storage)
        dform.title = "Preselectable Description Elements"
        dform.state = self.state
        hidden = set(self.state.mcplist)
        hidden.update(self.state.mutedChannels)

        nostcomps = set(self.state.avcplist) - set(self.state.stepComponents())
        dform.components = dict(
            (cp, False) for cp in nostcomps
            if cp not in hidden and not cp.startswith("__"))
        dform.components.update(
            dict((cp, True) for cp in self.state.acpgroup.keys()
                 if (cp not in hidden and not cp.startswith("__"))))

        cldsources = self.state.clientDataSources()
        hidden.update(
            set(cldsources)
            - set(self.state.motors)
            - set(self.state.ioregisters))

        dform.datasources = dict(
            (cp, False) for cp in self.state.avdslist
            if cp not in hidden and not cp.startswith("__"))
        dform.datasources.update(
            dict((cp, True) for cp in self.state.idsgroup.keys()
                 if (cp not in hidden and not cp.startswith("__"))))

        dform.createGUI()
        dform.exec_()
        if dform.dirty:
            self.__updateGroup(self.state.acpgroup, dform.components, True)
            self.__updateGroup(self.state.idsgroup, dform.datasources, True)
            self.resetViews.emit()
            self.dirty.emit()

    def __updateGroup(self, group, dct, dvalue=False):
        """ updates selection dictionary according to the given group

        :param group: component selection dictionary
        :type group: :obj:`dict` <:obj:`str`, :obj:`bool` or `None`>
        :param dct: component group
        :type dct: :obj:`dict` <:obj:`str`, :obj:`bool`>
        :param dvalue: default value
        :type dvalue: :obj:`bool`
        """
        ddsdict = self.state.ddsdict.keys()
        for k, st in group.items():
            if k not in dct.keys() \
               and k not in self.state.orderedchannels \
               and k not in ddsdict:
                group.pop(k)
        for k, st in dct.items():
            if k in group.keys():
                if st is False \
                   and k not in ddsdict:
                    group.pop(k)
            else:
                if st is True:
                    group[k] = dvalue

    def __createList(self, dct):
        """ provides a list of selected components from dictionary

        :param dct: component selection dictionary
        :type group: :obj:`dict` <:obj:`str`, :obj:`bool` or `None`>
        :returns:  list of selected components
        :rtype: :obj:`list` <:obj:`str`>
        """
        return [k for (k, st) in dct.items() if st is True]

    @Qt.pyqtSlot()
    def __props(self):
        """ changes component device properites
        """
        dform = PropertiesDlg(self.ui.storage)
#        dform.widget.labels = self.state.labels
        dform.widget.paths = self.state.labelpaths
        dform.widget.shapes = self.state.labelshapes
        dform.widget.links = self.state.labellinks
        dform.widget.canfailflags = self.state.labelcanfailflags
        dform.widget.types = self.state.labeltypes

        dform.available_names = list((set(self.state.labels.values())
                                      | set(self.state.avdslist)))
        dform.createGUI()
        dform.exec_()
        if dform.dirty:
            self.resetViews.emit()
            self.dirty.emit()

    @Qt.pyqtSlot()
    def __addTimer(self):
        """ adds timers
        """
        logger.debug("ADD Timer")
        if len(self.state.atlist) > len(self.__tWidgets) + 1:
            self.__appendTimer()
            self.state.timers.append("")
            self.reset()
            self.apply()
        logger.debug("ADD Timer end")

    def __appendTimer(self, connect=True):
        """ appends a new timer into the configuration

        :param connect: if currentIndexChanged should be connected to apply
        :type connect: :obj:`bool`
        """
        cb = Qt.QComboBox(self.ui.storage)
        self.__tWidgets.append(cb)
        if self.__layout is None:
            self.__layout = Qt.QHBoxLayout(self.ui.timerFrame)
            self.__layout.setContentsMargins(0, 0, 0, 0)
        self.__layout.addWidget(cb)
        cb.setEnabled(not self.__simplemode)
        if connect:
            cb.currentIndexChanged.connect(self.apply)

    @Qt.pyqtSlot()
    def __delTimer(self):
        """ removes the last timer from the configuration and widgets
        """
        logger.debug("delTimer")
        if self.__tWidgets:
            self.__removeTimer()
            self.state.timers.pop()
            self.reset()
            self.apply()
        logger.debug("delTimer end")

    def __removeTimer(self):
        """ removes the last timer from widgets
        """
        if self.__tWidgets:
            cb = self.__tWidgets.pop()
            cb.hide()
            try:
                cb.currentIndexChanged.disconnect(self.apply)
            except Exception:
                pass
            self.__layout.removeWidget(cb)
            cb.close()

    def reset(self):
        """ resets the storage tab
        """
        logger.debug("reset storage")
        self.disconnectSignals()
        try:
            self.updateForm()
        finally:
            self.connectSignals()
        logger.debug("reset storage ended")

    def __updateTimer(self, widget, nid):
        """ updates timer combobox list

        :param widget: timer combobox widget
        :type widget: :class:`taurus.qt.Qt.QComboBox`
        :param nid: timer index
        :type nid: :obj:`int`
        """
        widget.clear()
        mtimers = sorted(set(self.state.atlist))
        if self.state.timers is not None and len(self.state.timers) > nid:
            timer = self.state.timers[nid]
            if nid:
                mtimers = sorted(set(mtimers) - set(self.state.timers[:nid]))
        else:
            timer = ''
        widget.addItems(
            [str(tm) for tm in mtimers])
        cid = widget.findText(str(timer))
        if cid < 0:
            cid = 0
            if self.state.atlist:
                timer = self.state.atlist[nid]
                if self.state.timers and len(self.state.timers) > nid:
                    self.state.timers[nid] = timer
                elif nid == 0:
                    self.state.timers.append(timer)

        widget.setCurrentIndex(cid)

    @Qt.pyqtSlot()
    def updateScanID(self):
        """ updates scanid
        """
        logger.debug("updateForm storage")
        self.state.fetchEnvData({"ScanID": "scanID"})
        self.ui.fileScanIDSpinBox.setValue(self.state.scanID)
        logger.debug("updateForm storage ended")

    def updateForm(self):
        """ updates storage form
        """
        logger.debug("updateForm storage")
        # file group
        if self.state.scanDir is not None:
            self.ui.fileScanDirLineEdit.setText(self.state.scanDir)
        self.ui.fileScanIDSpinBox.setValue(self.state.scanID)
        self.ui.fileScanIDSpinBox.setEnabled(False)

        sfile = ""
        if self.state.scanFile:
            if isinstance(self.state.scanFile, (list, tuple)):
                sfile = ", ".join(self.state.scanFile)
            else:
                sfile = self.state.scanFile
            self.ui.fileScanLineEdit.setText(sfile)
        self.__updateTimer(self.ui.mntTimerComboBox, 0)
        while self.state.timers is not None and \
                len(self.state.timers) > len(self.__tWidgets) + 1:
            logger.debug("ADDING timer")
            self.__appendTimer(connect=False)
        while self.state.timers and \
                len(self.state.timers) < len(self.__tWidgets) + 1:
            logger.debug("removing timer")
            self.__removeTimer()
        for nid, widget in enumerate(self.__tWidgets):
            self.__updateTimer(widget, nid + 1)

        # measurement group
        if self.state.mntgrp is not None:
            self.ui.mntGrpComboBox.setEditText(self.state.mntgrp)
        self.ui.mntServerLineEdit.setText(self.state.door)

        # device group
        self.ui.devWriterLineEdit.setText(self.state.writerDevice)
        self.ui.devConfigLineEdit.setText(self.state.configDevice)

        # dynamic component group
#        self.ui.dcEnableCheckBox.setChecked(self.state.dynamicComponents)
        if self.state.dynamicLinks is not None:
            self.ui.dcLinksCheckBox.setChecked(self.state.dynamicLinks)
        if self.state.dynamicPath is not None:
            self.ui.dcPathLineEdit.setText(self.state.dynamicPath)

        # others group
        if self.state.appendEntry is not None:
            self.ui.othersEntryCheckBox.setChecked(self.state.appendEntry)

        logger.debug("updateForm storage ended")

    def __applyTimer(self, widget, nid):
        """ store timer into server state

        :param widget: timer combobox widget
        :type widget: :class:`taurus.qt.Qt.QComboBox`
        :param nid: timer index
        :type nid: :obj:`int`
        """
        timer = str(widget.currentText())
        if self.state.timers:
            if len(self.state.timers) <= nid:
                self.state.timers.append(timer)
            elif self.state.timers[nid] != timer:
                self.state.timers[nid] = timer

    @Qt.pyqtSlot()
    def __mntgrp_edited(self):
        """ updates application state on editing measurement group
        """
        logger.debug("mntgrp edited")
        current = str(self.ui.mntGrpComboBox.currentText()).lower()
        if current == self.state.mntgrp:
            return
        self.disconnectSignals()
        if not current:
            try:
                self.ui.mntGrpComboBox.setFocus()
            finally:
                self.connectSignals()
            return
        self.state.mntgrp = current
        if self.state.mntgrp not in self.state.avmglist:
            try:
                self.updateMntGrpComboBox()
            finally:
                self.connectSignals()
            self.apply()
        logger.debug("mntgrp edited end")

    @Qt.pyqtSlot()
    def __mntgrp_changed(self):
        """ updates application state on changing measurement group
        """
        logger.debug("mntgrp changed")
        current = str(self.ui.mntGrpComboBox.currentText()).lower()
        if current == self.state.mntgrp:
            return
        self.disconnectSignals()
        if not current:
            try:
                self.ui.mntGrpComboBox.setFocus()
            finally:
                self.connectSignals()
            return

        self.state.mntgrp = current
        if self.state.mntgrp not in self.state.avmglist:
            self.connectSignals()
            self.apply()
        else:
            try:
                self.state.storeData("mntGrp", self.state.mntgrp)
                self.state.fetchMntGrp()
            finally:
                self.connectSignals()
            self.resetAll.emit()
#        else:
#            self.connectSignals()
#            self.apply()
        logger.debug("mntgrp changed end")

    @Qt.pyqtSlot()
    def __mntgrp_deleted(self):
        """ updates application state on deleting measurement group
        """
        logger.debug("mntgrp deleted")
        replay = Qt.QMessageBox.question(
            self.ui.storage,
            "NXSelector: ",
            "Would you like to delete %s Measurement Group? "
            % self.ui.mntGrpComboBox.currentText(),
            Qt.QMessageBox.Yes | Qt.QMessageBox.No)
        if replay == Qt.QMessageBox.Yes:
            self.disconnectSignals()
            try:
                self.state.deleteMntGrp(
                    str(self.ui.mntGrpComboBox.currentText()).lower())
            finally:
                self.connectSignals()
            self.resetAll.emit()
        logger.debug("mntgrp deleted end")

    @Qt.pyqtSlot()
    def __setDir(self):
        """ sets scan directory
        """
        dirname = str(Qt.QFileDialog.getExistingDirectory(
            self.ui.storage,
            "Scan Directory",
            self.state.scanDir))
        if str(dirname) and str(dirname) != str(self.state.scanDir):
            self.ui.fileScanDirLineEdit.setText(dirname)
            self.state.scanDir = str(dirname)
            self.apply()

    @Qt.pyqtSlot()
    def __dirty(self):
        """ sets dirty flag
        """
        self.dirty.emit()

    @Qt.pyqtSlot()
    def __dirChanged(self):
        """ updates application state on a scan directory change
        """
        dirname = str(self.ui.fileScanDirLineEdit.text())
        if self.state.scanDir != dirname:
            self.apply()

    @Qt.pyqtSlot()
    def __fileChanged(self):
        """ updates application state on a scan file change
        """
        fnames = self.__fileNames(False)
        if json.dumps(self.state.scanFile) != json.dumps(fnames):
            self.apply()

    def __fileNames(self, message=True):
        """ corrects the scan file name lists

        :param message: message on false
        :type message: :obj:`str`
        :returns: scan file name or a list of scan file names
        :rtype: :obj:`str` or :obj:`list` <:obj:`str`>
        """
        files = str(self.ui.fileScanLineEdit.text())
        sfiles = files.replace(';', ' ').replace(',', ' ').split()
        nxsfiles = []
        for idx, f in enumerate(sfiles):
            if f.split(".")[-1] == 'nxs':
                nxsfiles.append(idx)

        if message and len(nxsfiles) > 1 \
                and self.state.writerDevice != str(self.__moduleLabel):
            Qt.QMessageBox.warning(
                self.ui.storage,
                "To many 'nxs' scan files",
                "Only %s will be used." % (sfiles[nxsfiles[0]]))

            for f in reversed(nxsfiles[1:]):
                sfiles.pop(f)
        return sfiles[0] if len(sfiles) == 1 else sfiles

    @Qt.pyqtSlot()
    def apply(self):
        """ stores form values into server state object
        """
        logger.debug("updateForm apply")
        self.disconnectSignals()
        if not str(self.ui.mntGrpComboBox.currentText()):
            try:
                self.ui.mntGrpComboBox.setFocus()
            finally:
                self.connectSignals()
            return
        try:
            self.state.mntgrp = str(
                self.ui.mntGrpComboBox.currentText()).lower()

            logger.debug("apply Timers")
            self.__applyTimer(self.ui.mntTimerComboBox, 0)
            for nid, widget in enumerate(self.__tWidgets):
                self.__applyTimer(widget, nid + 1)
            logger.debug("apply Timers ended")

            self.state.door = str(self.ui.mntServerLineEdit.text())

            # device group
            self.state.writerDevice = str(self.ui.devWriterLineEdit.text())
            self.state.configDevice = str(self.ui.devConfigLineEdit.text())

            self.state.scanDir = str(self.ui.fileScanDirLineEdit.text())
    #        self.state.scanID = int(self.ui.fileScanIDSpinBox.value())
            self.state.scanFile = self.__fileNames()

            # dynamic component group
            self.state.dynamicComponents = True
            self.state.dynamicLinks = self.ui.dcLinksCheckBox.isChecked()
            self.state.dynamicPath = str(self.ui.dcPathLineEdit.text())

            # others group
            self.state.appendEntry = self.ui.othersEntryCheckBox.isChecked()
        finally:
            self.connectSignals()

        self.dirty.emit()
        self.resetViews.emit()
        logger.debug("updateForm apply ended")



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# info about tango servers

"""  info about tango servers """

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from taurus.qt.qtgui.util.ui import UILoadable

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


@UILoadable(with_ui='ui')
class InfoDlg(Qt.QDialog):
    """  info about tango servers """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QDialog.__init__(self, parent)
        self.loadUi()
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = None

    def createGUI(self):
        """ creates GUI
        """
        if self.state:
            self.ui.writerLabel.setText(self.state.writerDevice)
            self.ui.configLabel.setText(self.state.configDevice)
            self.ui.doorLabel.setText(self.state.door)
            self.ui.selectorLabel.setText(str(
                self.state.server if self.state.server else 'module'))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# editable list dialog

"""  editable list dialog """

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from taurus.qt.qtgui.util.ui import UILoadable

from .EdDataDlg import EdDataDlg

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


class EdListDlg(Qt.QDialog):
    """  editable list dialog
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QDialog.__init__(self, parent)
        #: (:class:`nxselector.EdListWg.EdListWg`)  dialog widget
        self.widget = EdListWg(self)
        #: (:obj:`bool`) simple view
        self.simple = False
        #: (:obj:`bool`) dirty flag
        self.dirty = False
        #: (:obj:`list` <:obj:`str`>) available names
        self.available_names = None
        #: (:obj:`list` <:obj:`str`>) table headers
        self.headers = ["Name", "Value"]
        #: (:obj:`list` <:obj:`str`>) disable names
        self.disable = []

    def createGUI(self):
        """ creates widget GUI
        """
        self.widget.simple = self.simple
        self.widget.available_names = self.available_names
        self.widget.headers = self.headers
        self.widget.disable = self.disable
        self.widget.createGUI()
        layout = Qt.QHBoxLayout()
        layout.addWidget(self.widget)
        self.setLayout(layout)

        self.widget.ui.closeButtonBox.button(
            Qt.QDialogButtonBox.Close).clicked.connect(self.accept)
        self.widget.ui.closePushButton.show()
        self.widget.dirty.connect(self.__setDirty)

    @Qt.pyqtSlot()
    def __setDirty(self):
        """ set dirty flag """
        self.dirty = True


@UILoadable(with_ui='ui')
class EdListWg(Qt.QWidget):
    """  editable list widget
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QWidget.__init__(self, parent)
        self.loadUi()
        #: (:obj:`bool`) simple view flag
        self.simple = False
        #: (:obj:`dict` <:obj:`str`, `any`>) data (name, value) dictionary
        self.record = {}
        #: (:obj:`list` <:obj:`str`>) available names
        self.available_names = None
        #: (:obj:`list` <:obj:`str`>) table headers
        self.headers = ["Name", "Value"]
        #: (:obj:`list` <:obj:`str`>) disable names
        self.disable = []

    def createGUI(self):
        """ creates widget GUI
        """
        if hasattr(self.ui, "addPushButton"):
            self.ui.addPushButton.clicked.disconnect(self.__add)
            self.ui.editPushButton.clicked.disconnect(self.__edit)
            self.ui.tableWidget.itemDoubleClicked.disconnect(self.__edit)
            self.ui.removePushButton.clicked.disconnect(self.__remove)

        self.ui.closePushButton = self.ui.closeButtonBox.button(
            Qt.QDialogButtonBox.Close)

        self.ui.closeButtonBox.button(Qt.QDialogButtonBox.Close).hide()
        if not hasattr(self.ui, "addPushButton"):
            self.ui.addPushButton = self.ui.addEditRemoveButtonBox.addButton(
                "&Add", Qt.QDialogButtonBox.ActionRole)
            self.ui.editPushButton = self.ui.addEditRemoveButtonBox.addButton(
                "&Edit", Qt.QDialogButtonBox.ActionRole)
            self.ui.removePushButton = \
                self.ui.addEditRemoveButtonBox.addButton(
                    "&Remove", Qt.QDialogButtonBox.ActionRole)

        if self.record:
            item = sorted(self.record.keys())[0]
        else:
            item = None
        self.__populateTable(item)
        self.ui.addPushButton.clicked.connect(self.__add)
        self.ui.editPushButton.clicked.connect(self.__edit)
        self.ui.tableWidget.itemDoubleClicked.connect(self.__edit)
        self.ui.removePushButton.clicked.connect(self.__remove)

    def __populateTable(self, selected=None):
        """ populate table records

        :param selected: name of selected item
        :type selected: :obj:`str`
        """
        self.ui.tableWidget.clear()
        sitem = None
        self.ui.tableWidget.setSortingEnabled(False)
        names = sorted(self.record.keys())
        self.ui.tableWidget.setRowCount(len(names))
        self.ui.tableWidget.setColumnCount(len(self.headers))
        self.ui.tableWidget.setHorizontalHeaderLabels(self.headers)
        for row, name in enumerate(names):
            enable = True
            if self.available_names is not None and\
                    name not in self.available_names:
                enable = False
            item = Qt.QTableWidgetItem(name)
            if self.available_names is not None:
                if enable is False:
                    flags = item.flags()
                    flags &= ~Qt.Qt.ItemIsEnabled
                    item.setFlags(flags)
            if selected is not None and selected == name:
                sitem = item
            self.ui.tableWidget.setItem(row, 0, item)
            value = self.record[name]
            item = Qt.QTableWidgetItem(str(value))
            if name in self.disable:
                flags = item.flags()
                flags &= ~Qt.Qt.ItemIsEnabled
                item.setFlags(flags)
            self.ui.tableWidget.setItem(row, 1, item)
        self.ui.tableWidget.resizeColumnToContents(0)
        self.ui.tableWidget.setSelectionBehavior(
            Qt.QAbstractItemView.SelectRows)
        self.ui.tableWidget.setSelectionMode(
            Qt.QAbstractItemView.SingleSelection)
        self.ui.tableWidget.horizontalHeader(
        ).setStretchLastSection(True)
        self.ui.tableWidget.setEditTriggers(
            Qt.QAbstractItemView.NoEditTriggers)
        if sitem is not None:
            sitem.setSelected(True)
            self.ui.tableWidget.setCurrentItem(sitem)

    def __currentName(self):
        """ provides currently selected name

        :returns: currently selected name
        :rtype: :obj:`str`
        """
        name = None
        row = self.ui.tableWidget.currentRow()
        skeys = sorted(self.record.keys())
        if len(skeys) > row and row >= 0:
            name = skeys[row]
        return name

    @Qt.pyqtSlot()
    def __add(self):
        """ adds a new record into the table
        """
        dform = EdDataDlg(self)
        dform.simple = self.simple
        dform.headers = self.headers
        dform.available_names = list(
            set(self.available_names) - set(self.disable))
        dform.available_tips = True
        dform.createGUI()
        if dform.exec_():
            if dform.name in self.disable:
                Qt.QMessageBox.information(
                    self, "Selector in Simple/User Mode",
                    "This data cannot be edited")
                return

            self.record[dform.name] = dform.value
            self.__populateTable()
            self.dirty.emit()
        dform.setParent(None)

    @Qt.pyqtSlot()
    def __edit(self):
        """ edits the current record in the table
        """
        name = self.__currentName()
        if name in self.disable:
            Qt.QMessageBox.information(
                self, "Selector in Simple/User Mode",
                "This data cannot be edited")
            return
        dform = EdDataDlg(self)
        dform.simple = self.simple
        dform.available_names = self.available_names
        if name:
            dform.name = name
            dform.value = self.record[name]
        dform.headers = self.headers
        dform.createGUI()
        if name:
            dform.ui.nameComboBox.setEnabled(False)
        if dform.exec_():
            self.record[dform.name] = dform.value
            self.__populateTable()
            self.dirty.emit()
        dform.setParent(None)

    @Qt.pyqtSlot()
    def __remove(self):
        """ removes the current record from the table
        """
        name = self.__currentName()
        if name in self.disable:
            Qt.QMessageBox.information(
                self, "Selector in Simple Mode",
                "This data cannot be removed")
            return
        if name not in self.record:
            return

        if Qt.QMessageBox.question(
                self, "Removing Data",
                "Would you like  to remove '%s'?" % name,
                Qt.QMessageBox.Yes | Qt.QMessageBox.No,
                Qt.QMessageBox.Yes) == Qt.QMessageBox.No:
            return
        self.record.pop(name)
        self.dirty.emit()
        self.__populateTable()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# editable data dialog

"""  editable data dialog """

import json
import sys

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from taurus.qt.qtgui.util.ui import UILoadable

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


if sys.version_info > (3,):
    unicode = str


@UILoadable(with_ui='ui')
class EdDataDlg(Qt.QDialog):
    """ editable data dialog
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QDialog.__init__(self, parent)
        self.loadUi()
        #: (:obj:`bool`) dialog simple version
        self.simple = False
        #: (:obj:`str`) data name
        self.name = ''
        #: (:obj:`str` or `any`) data values
        self.value = ''
        #: (:obj:`bool`) if value is string
        self.isString = True
        #: (:obj:`list` <:obj:`str`> ) table headers
        self.headers = []
        #: (:obj:`list` <:obj:`str`> ) available data names
        self.available_names = None

    def setText(self, text):
        """ sets text into name QComboBox

        :param text: data name
        :type text: :obj:`str`
        """
        if str(text) not in self.available_names:
            self.ui.nameComboBox.addItem(str(text))
        ind = self.ui.nameComboBox.findText(str(text))
        self.ui.nameComboBox.setCurrentIndex(ind)
        self.ui.nameComboBox.setEditText(str(text))

    def createGUI(self):
        """ creates widget GUI
        """
        if self.available_names:
            self.ui.nameComboBox.clear()
            for an in self.available_names:
                self.ui.nameComboBox.addItem(str(an))
        if len(self.headers) > 0:
            self.ui.nameLabel.setText(str(self.headers[0]))
            if len(self.headers) > 1:
                self.ui.valueLabel.setText(str(self.headers[1]))
        if self.simple:
            self.ui.stringCheckBox.hide()
        else:
            self.isString = isinstance(self.value,
                                       (str, unicode))
        self.ui.stringCheckBox.setChecked(self.isString)
        self.setText(str(self.name))
        self.ui.valueTextEdit.setText(str(str(self.value)))

#        if self.available_names:
#            completer = Qt.QCompleter(self.available_names, self)
#            self.ui.nameLineEdit.setCompleter(completer)

        self.ui.buttonBox.accepted.connect(self.accept)
        self.ui.buttonBox.rejected.connect(self.reject)

    @Qt.pyqtSlot()
    def accept(self):
        """ checks if dialog is filled in correctly and accepts it
        """
        self.name = unicode(self.ui.nameComboBox.currentText())
        self.isString = self.ui.stringCheckBox.isChecked()
        self.value = unicode(self.ui.valueTextEdit.toPlainText())
        if not self.isString and not self.simple:
            try:
                self.value = json.loads(self.value)
            except Exception:
                pass

        if not self.name:
            Qt.QMessageBox.warning(self, "Wrong Data", "Empty data name")
            self.ui.nameComboBox.setFocus()
            return
        Qt.QDialog.accept(self)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# editable list of component groups

"""  editable list dialog """

import sys

try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from taurus.qt.qtgui.util.ui import UILoadable
# from taurus.qt.qtgui.panel import TaurusModelChooser
from taurus.qt.qtgui.panel import TaurusModelSelectorTree
from taurus.core.taurusbasetypes import TaurusElementType
import taurus

# from .Views import OneTableView
# from .Element import GElement, CP, DS
# from .ElementModel import ElementModel

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


if sys.version_info > (3,):
    unicode = str


class SourceLineEdit(Qt.QLineEdit):

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QLineEdit.__init__(self, parent)
        self.setAcceptDrops(True)

    def dropEvent(self, event):
        """ set the attribute source from the drop event

        :param event: drop event
        :type event: :class:`Qt.QEvent`
        """
        if event.mimeData().hasUrls():
            # urlcount = len(event.mimeData().urls())
            url = event.mimeData().urls()[0]
            source = str(url.toString())
            if source.startswith("tango://"):
                source = source[8:]
            self.setText(source)
            event.accept()


@UILoadable(with_ui='ui')
class AddDataSourceDlg(Qt.QDialog):
    """  editable other device list dialog
    """

    def __init__(self, parent=None):
        """ constructor

        :param parent: parent object
        :type parent: :class:`taurus.qt.Qt.QObject`
        """
        Qt.QDialog.__init__(self, parent)
        self.loadUi()
        #: (:obj:`str`) datasource name
        self.name = ""
        #: (:obj:`str`) full name of tango device attribute
        self.source = ""
        #: (:obj:`list` <:obj:`str`>) existing datasources
        self.datasources = []
        #: (:obj:`str`) group title
        self.title = "Create DataSource"
        #: (:obj:`list` <:obj:`str`>) forbidden name characters
        self.__chars = ["/", ":", "#", "->"]

    def createGUI(self):
        """ creates GUI
        """
        self.setWindowTitle(self.title)
        try:
            host = taurus.Factory('tango').getAuthority().getFullName()
        except Exception:
            try:
                host = taurus.Factory('tango').getDatabase().getFullName()
            except Exception:
                host = None

        self.ui.tree = TaurusModelSelectorTree(
            selectables=[TaurusElementType.Attribute],
            buttonsPos=Qt.Qt.BottomToolBarArea)
        self.ui.tree.setModel(host)
        self.ui.tree.treeView().setSelectionMode(
            Qt.QAbstractItemView.SingleSelection)
        self.ui.tree._toolbar.hide()
        self.ui.sourceLineEdit = SourceLineEdit(self)
        self.ui.sourceLineEdit.setToolTip(
            "Drop an attribute from the above tree")
        if host and host.startswith("tango://"):
            host = host[8:]
        self.ui.hostLineEdit.setText(str(host or ""))
        gridLayout = Qt.QGridLayout(self.ui.widget)
        gridLayout.addWidget(self.ui.tree, 0, 0, 1, 1)
        self.ui.gridLayout.addWidget(self.ui.sourceLineEdit, 1, 1, 2, 2)
        self.ui.hostLineEdit.editingFinished.connect(self.__updatehostname)

    @Qt.pyqtSlot()
    def __updatehostname(self):
        """ update the tango host name
        """
        host = str(self.ui.hostLineEdit.text())
        if ":" not in host:
            host = host + ":10000"
            self.ui.hostLineEdit.setText(host)
        self.ui.tree.setModel("tango://%s" % host)

    def accept(self):
        """ updates class variables with the form content
        """
        self.name = unicode(self.ui.nameLineEdit.text())
        self.source = unicode(self.ui.sourceLineEdit.text())

        if not self.name:
            Qt.QMessageBox.warning(
                self, "Wrong form input", "Empty datasource name")
            self.ui.nameLineEdit.setFocus()
            return

        name = self.name
        for ch in self.__chars:
            if ch in name:
                name = name.replace(ch, "_")
        if self.name != name:
            self.ui.nameLineEdit.setText(name)
            self.name = name
            Qt.QMessageBox.warning(
                self, "Wrong form input",
                "Datasource name cannot contain any of '%s'" %
                "', '".join(self.__chars))
            self.ui.nameLineEdit.setFocus()
            return

        if self.name in self.datasources:
            Qt.QMessageBox.warning(
                self, "Wrong form input",
                "DataSource %s already exists" % self.name)
            self.ui.nameLineEdit.setFocus()
            return

        if not self.source:
            Qt.QMessageBox.warning(
                self, "Wrong form input", "Empty attribute name")
            self.ui.sourceLineEdit.setFocus()
            return

        Qt.QDialog.accept(self)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# user data tab

""" user data tab """


try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from .EdListDlg import EdListWg


import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)


class Data(Qt.QObject):
    """ User data tab widget
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) dirty signal
    dirty = Qt.pyqtSignal()

    def __init__(self, ui, state=None, simpleMode=False):
        """ constructor

        :param ui: ui instance
        :type ui: :class:`taurus.qt.qtgui.util.ui.__UI`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param simpleMode: if simple display mode
        :type simpleMode: :obj:`bool`
        """
        Qt.QObject.__init__(self)
        #: (:class:`taurus.qt.qtgui.util.ui.__UI`) ui instance
        self.ui = ui
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = state
        #: (:class:`taurus.qt.Qt.QLayout`)
        self.glayout = None
        #: (:obj:`bool`) if simple view mode
        self.__simpleMode = simpleMode
        #: (:class:`nxsselector.EdListWg.EdListWg`) table editing widget
        self.form = EdListWg(self.ui.data)

    def createGUI(self):
        """ creates widget GUI
        """
        self.ui.data.hide()

        if self.glayout:
            child = self.glayout.takeAt(0)
            while child:
                self.glayout.removeItem(child)
                if isinstance(child, Qt.QWidgetItem):
                    self.glayout.removeWidget(child.widget())
                child = self.glayout.takeAt(0)
            self.form.dirty.disconnect(self.__setDirty)
        else:
            self.glayout = Qt.QHBoxLayout(self.ui.data)

        if self.form:
            self.form.setParent(None)

        if self.__simpleMode:
            self.form.disable = self.state.admindata
        self.form.record = self.state.datarecord
        names = self.state.clientRecords()
        logger.debug("NAMES: %s " % names)
        self.form.available_names = names
        self.form.createGUI()
        self.glayout.addWidget(self.form)
        self.ui.data.update()
        if self.ui.tabWidget.currentWidget() == self.ui.data:
            self.ui.data.show()
        self.form.dirty.connect(self.__setDirty)

    def reset(self):
        """ recreates widget GUI
        """
        self.createGUI()

    @Qt.pyqtSlot()
    def __setDirty(self):
        """ emits the `dirty` signal
        """
        self.dirty.emit()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2014-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# descriptions tab

""" descriptions tab """


try:
    from taurus.external.qt import Qt
except Exception:
    from taurus.qt import Qt

from .Element import CPElement
from .ElementModel import ElementModel
from .Views import CheckerView
from .DynamicTools import DynamicTools

import logging
#: (:obj:`logging.Logger`) logger object
logger = logging.getLogger(__name__)

#: (:obj:`int`) component type enum
OPTIONAL, MANDATORY, OTHERS = range(3)


class Descriptions(Qt.QObject):
    """ descriptions tab widget
    """

    #: (:class:`taurus.qt.Qt.pyqtSignal`) component checked signal
    componentChecked = Qt.pyqtSignal()

    def __init__(self, ui, state=None, userView=CheckerView,
                 mandUserView=CheckerView, rowMax=0, fontSize=11):
        """ constructor

        :param ui: ui instance
        :type ui: :class:`taurus.qt.qtgui.util.ui.__UI`
        :param state: server state
        :type state: :class:`nxsselector.ServerState.ServerState`
        :param userView: widget for description optional components
        :type userView: :class:`taurus.qt.Qt.QWidget`
        :param mandUserView: widget for description mandatory components
        :type mandUserView: :class:`taurus.qt.Qt.QWidget`
        :param rowMax: maximal row number in component column view
        :type rowMax: :obj:`int`
        :param fontSize: font size in component column view
        :type fontSize: :obj:`int`
        """

        Qt.QObject.__init__(self)
        #: (:class:`taurus.qt.qtgui.util.ui.__UI`) ui instance
        self.ui = ui
        #: (:class:`nxsselector.ServerState.ServerState`) server state
        self.state = state
        #: (:class:`taurus.qt.Qt.QWidget`) \
        #:     widget for description optional components
        self.userView = userView
        #: (:class:`taurus.qt.Qt.QWidget`) \
        #:     widget for description mandatory components
        self.mandUserView = mandUserView
        #: (:obj:`int`) font size for in component column view
        self.fontSize = fontSize
        #: (:obj:`int`) maximal row number in component column view
        self.rowMax = rowMax

        #: (:obj:`list` <:class:`nxsselector.Element.CPElement`>) \
        #:    group of optional component elements
        self.agroup = []
        #: (:obj:`list` <:class:`nxsselector.Element.CPElement`>) \
        #:    group of other component elements
        self.igroup = []
        #: (:obj:`list` <:class:`nxsselector.Element.CPElement`>) \
        #:    group of mandator component elements
        self.mgroup = []

        #: (:obj:`list` <:class:`taurus.qt.Qt.QFrame`>) \
        #:  list of component group frames
        self.mframes = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QLayout`>) \
        #:  list of group layouts
        self.group_layouts = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QLayout`>) \
        #:  list of widget layouts
        self.auto_layouts = []
        #: (:obj:`list` <:class:`taurus.qt.Qt.QGroupBox`>) \
        #:  list of component group boxes
        self.groupboxes = []
        #: (:obj:`dict` <:obj:`int`, :class:`taurus.qt.Qt.QWidget`>) \
        #:    (OPTIONAL, MANDATORY, OTHERS) component views
        self.views = {}
        #: (:obj:`list` <:class:`nxsselector.ElementModel.ElementModel`>) \
        #:    element model list
        self.models = []
        #: (:class:`taurus.qt.Qt.QLayout`) component layout
        self.__cplayout = None
        #: (:class:`taurus.qt.Qt.QLayout`) other component layout
        self.__dslayout = None
        #: (:class:`taurus.qt.Qt.QLayout`) main layout
        self.__mainlayout = None

    def updateGroups(self):
        """ updates component/datasource selection state
        """

        self.agroup = []
        for cp in self.state.acpgroup.keys():
            self.agroup.append(
                CPElement(cp, self.state,
                          group=self.state.acpgroup))

        self.mgroup = []
        mcpgroup = {}
        for cp in self.state.mcplist:
            mcpgroup[cp] = True
        for cp in mcpgroup.keys():
            self.mgroup.append(
                CPElement(cp, self.state, group=mcpgroup))

        self.igroup = []
        for ds in self.state.idsgroup.keys():
            self.igroup.append(
                CPElement(ds, self.state,
                          group=self.state.idsgroup))

    def __clearFrames(self):
        """ clears component frames
        """
        DynamicTools.cleanupObjects(self.models, "model")
        if self.views:
            views = list(self.views.values())
            DynamicTools.cleanupWidgets(views, "views")
            self.views = {}
        DynamicTools.cleanupObjects(self.auto_layouts, "auto")
        DynamicTools.cleanupWidgets(self.groupboxes, "groupbox")
        DynamicTools.cleanupObjects(self.group_layouts, "group")
        DynamicTools.cleanupFrames(self.mframes, "frames")
        DynamicTools.cleanupLayoutWithItems(self.__cplayout)
        DynamicTools.cleanupLayoutWithItems(self.__dslayout)
        DynamicTools.cleanupLayoutWithItems(self.__mainlayout)
        self.__cplayout = None
        self.__dslayout = None

    def createGUI(self):
        """ creates widget GUI
        """

        self.ui.descriptions.hide()
        if self.__cplayout and self.__dslayout:
            self.__clearFrames()
        self.__mainlayout = Qt.QHBoxLayout(self.ui.descriptions)
        self.__cplayout = Qt.QVBoxLayout()
        self.__dslayout = Qt.QVBoxLayout()
        self.__mainlayout.addLayout(self.__cplayout)
        self.__mainlayout.addLayout(self.__dslayout)
        self.__mainlayout.setStretchFactor(self.__cplayout, 3)
        self.__mainlayout.setStretchFactor(
            self.__dslayout, 1 if self.igroup else 0)

        self.views[OTHERS] = self.__addView(
            "Other Optional", self.userView, self.rowMax,
            self.__dslayout, not self.igroup)

        la = len(self.agroup)
        lm = len(self.mgroup)
        if la + lm:
            la, lm = [float(la) / (la + lm), float(lm) / (la + lm)]

        self.views[OPTIONAL] = self.__addView(
            "Optional", self.userView, max(1, int(la * (self.rowMax - 2))))
        self.views[MANDATORY] = self.__addView(
            "Mandatory", self.mandUserView,
            max(1, int(lm * (self.rowMax - 2))))

        self.ui.descriptions.update()
        if self.ui.tabWidget.currentWidget() == self.ui.descriptions:
            self.ui.descriptions.show()

    def __addView(self, label, view, rowMax, layout=None, hide=False):
        """ adds component group frame views

        :param label: component group label
        :type label: :obj:`str`
        :param view: component view widget
        :type view: :class:`taurus.qt.Qt.QWidget`
        :param rowMax: maximal row number in component column view
        :type rowMax: :obj:`int`
        :param layout: component group frame layout
        :type layout: :class:`taurus.qt.Qt.QLayout`
        :param hide: if frame should be hidden
        :type hide: :obj:`bool`
        """
        if layout is None:
            layout = self.__cplayout
        mframe = Qt.QFrame(self.ui.descriptions)
        mframe.setFrameShape(Qt.QFrame.StyledPanel)
        mframe.setFrameShadow(Qt.QFrame.Raised)
        self.mframes.append(mframe)
        layout_groups = Qt.QHBoxLayout(mframe)
        self.group_layouts.append(layout_groups)

        mgroup = Qt.QGroupBox(mframe)
        mgroup.setTitle(label)
        self.groupboxes.append(mgroup)
        layout_auto = Qt.QGridLayout(mgroup)
        self.auto_layouts.append(layout_auto)
        mview = view(mgroup)
        if hasattr(mview, "rowMax"):
            mview.rowMax = rowMax
        if hasattr(mview, "fontSize"):
            mview.fontSize = self.fontSize
        if hasattr(mview, 'dmapper'):
            mview.dmapper = None

        layout_auto.addWidget(mview, 0, 0, 1, 1)
        layout_groups.addWidget(mgroup)

        layout.addWidget(mframe)
        if hide:
            mframe.hide()
        else:
            mframe.show()
        return mview

    def setModels(self):
        """ sets view models
        """

        md = ElementModel(self.agroup)
#        md.enable = False
        self.models.append(md)
        self.views[OPTIONAL].setModel(md)
        md.componentChecked.connect(self.__componentChecked)

        md = ElementModel(self.mgroup)
        md.enable = False
        self.models.append(md)
        self.views[MANDATORY].setModel(md)
        md.componentChecked.connect(self.__componentChecked)

        md = ElementModel(self.igroup)
#        md.enable = False
        self.models.append(md)
        self.views[OTHERS].setModel(md)
        md.componentChecked.connect(self.__componentChecked)

    @Qt.pyqtSlot()
    def __componentChecked(self):
        """ emits the component checked signal
        """

        self.componentChecked.emit()

    def updateViews(self):
        """ resets frame views
        """

        for vw in self.views.values():
            vw.reset()

    def reset(self):
        """ resets whole content of frame views
        """

        self.updateGroups()
        self.createGUI()
        self.setModels()
        self.updateViews()
